/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set ts=2 sw=4 sts=2 et cindent: */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the MICROB EAL package.
 *
 * The Initial Developer of the Original Code is Nokia Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contact: Oleg Romashin <oleg.romashin@nokia.com>
 *
 * ***** END LICENSE BLOCK ***** */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gmozillaengine.h"
#include "gmozillamarshal.h"
#include "gmozillaweb.h"
#include "gmozillatransferitem.h"
#include "common.h"
#include "gmozillaconnectivity.h"
#include "gmozillahistorylist.h"
#include "gmozillasupport.h"
#include "gtkmicrob_context.h"

#include <sys/time.h>
#include <time.h>
#include <string.h>
#include <malloc.h>

#ifdef USE_OPENSSL
#include <openssl/x509.h>
#endif

#define SINGLE_LINE_WIDGET_NAME "sinleline"
#ifdef USE_HILDON2007
#include <hildon/hildon-helper.h>
#ifdef HILDON_2_2
#include <hildon/hildon-touch-selector.h>
#include <hildon/hildon-touch-selector-entry.h>
#include <hildon/hildon-button.h>
#include "browser-translations.h"
#endif
#else
#ifdef USE_HILDON
#ifndef USE_OS2006
#include <hildon-widgets/hildon-finger.h>
#endif
#endif
#endif

#define GTK_MOZ_EMBED_STATUS_PROXY_FAILED 2152398920U

#define NODE_SCROLL_TIMEOUT 300
#define MAXIMUM_DIALOG_WIDTH 750

#define LOCALLY_CACHED_PAGE_URL_PREFIX "wyciwyg:"
#define LOCALLY_CACHED_PAGE_URL_PREFIX_LEN 8

// global transfer item.
gpointer global_last_transfer_item_object = NULL;
struct timeval current_time_press, former_time_press, result; //double click emulation timers

GWebEngineEncoding g_mozilla_engine_get_encoding (GtkMozEmbed *);
static void g_mozilla_engine_notify_memory (GMozillaEngine *self);
static gboolean show_menu_cb (gpointer data);

// class and instance initialization
enum
{
    TITLE_CHANGED,                /** < When the title is changed */
    URL_CHANGED,                  /** < When the url is changed */
    LOADING_PROGRESS,             /** < Loagind progress signal */
    START_LOADING,                /** < Start loading signal */
    FINISHED_LOADING,             /** < Finished loading signal */
    ENCODING,                     /** < Encoding signal */
    DOWNLOAD_REQUEST,             /** < Download request signal */
    AUTHENTIFICATION_REQUIRED,    /** < When an authentification is required */
    ALERT,                        /** < When an alert is required */
    CONFIRM,                      /** < When a confirm dialog is required */
    CONFIRM_EX,
    PROMPT,                       /** < When a prompt dialog is required */
    SELECT,                       /** < When a select dialog is required */
    MAILTO,                       /** < When a mail signal is required */
    UNKNOWN_PROTOCOL,             /** < Unknown protocol signal is required */
    SECURITY_MODE_CHANGED,        /** < When security mode is changed */
    WINDOW_REQUEST,               /** < When a new window is required */
    CLOSE_WINDOW,                 /** < Close window signal */
    RAISE_REQUEST,                /** < Raise request signal */
    LOWER_REQUEST,                /** < Lower request signal */
    UPLOAD_DIALOG,                /** < When an upload is required */
    FILE_UPLOAD_STARTED,          /** < When an upload is started */
    FILE_UPLOAD_PROGRESS,         /** < When an upload is in progress */
    FILE_UPLOAD_FINISHED,         /** < when an upload is finished */
    WEBENGINE_POPUP_MENU,         /** < When a pop up window is required */
    EXTERNAL_ACTION,              /** < When an external action is required */
    MODAL_DIALOG,                 /** < When a modal dialog is required */
    FRAME_STACKING_CHANGED,       /** < When a focus frame is changed */
    MODAL_CERTIFICATE_DIALOG,     /** < When a certificate dialog is required */
    ASK_COOKIE,                   /** < When an ask cookie dialog is required */
    CERTIFICATE_DIALOG,           /** < When a certificate dialog is required */
    CERTIFICATE_PASSWORD_DIALOG,  /** < When a certificate dialog requires a password */
    CERTIFICATE_DETAILS,          /** < When a certificate details dialog is required */
    HISTORY_ADDED,                /** < When a url is added into history */
    SELECT_MATCH_SIGNAL,
    ON_SUBMIT_SIGNAL,
    ICON_SIGNAL,
    RSS_SIGNAL,
    KEY_EVENT_SIGNAL,
    ON_SERVICE_NOTIFY,
    PREF_WINDOW_SIZE,             /** < Window resize request */
    OPEN_URI,                     /** < Window resize request */
    MOZ_PAINT_NOTIFY,
    SHM_BUFFER_INFO,
    NOTIFICATION_DIALOG,
    HISTORY_CHANGED,
    META_REFRESH,
    ADD_SEARCH_PROVIDER,
    MOZILLA_ENGINE_LAST_SIGNAL	  /** < Last signal used */
};

enum
{
    CTX_USER,
    CTX_DOM,
    CTX_MOUSE
};

guint g_mozilla_engine_signals[MOZILLA_ENGINE_LAST_SIGNAL] = { 0 };

// class stuff
static void g_mozilla_engine_class_init (GMozillaEngineClass *);
static void g_mozilla_engine_finalize (GObject * instance);
static void g_mozilla_engine_destroy (GMozillaEngine *);
static void load_finished_cb (GtkMozEmbed *embed, GMozillaEngine *self);
static gboolean g_mozilla_engine_update_context_data (GMozillaEngine *self, guint ctx_type, GtkMozEmbed *embed, gpointer event);
static gboolean g_mozilla_engine_signals_disconnect(GMozillaEngine *self);
static void g_mozilla_unregister_im_context(GMozillaEngine *self);
GtkIMContext* register_im_context(GMozillaEngine *self);
static gboolean vkb_handle_action(GMozillaEngine *self, guint moz_embed_ctx, GdkEventKey * event);

static void user_iteraction_happens(GMozillaEngine *self)
{
   self->user_iteraction_happens = TRUE;
}

static gboolean
protocol_is_local(const gchar *uri)
{
    int i;
    if (!*uri)
        return FALSE;

    if ('/'==uri[0])
        return TRUE;

    /* http(s) is not local */
    if ((uri[i=0]=='h' || uri[i]=='H') &&
        (uri[++i]=='t' || uri[i]=='T') &&
        (uri[++i]=='t' || uri[i]=='T') &&
        (uri[++i]=='p' || uri[i]=='P') &&
        (uri[++i]==':' ||
         ((uri[i]=='s' || uri[i]=='S') &&
          uri[++i]==':')))
        return FALSE;

    /* ftp is not local */
    if ((uri[i=0]=='f' || uri[i]=='F') &&
        (uri[++i]=='t' || uri[i]=='T') &&
        (uri[++i]=='p' || uri[i]=='P') &&
        (uri[++i]==':'))
        return FALSE;

    /* Anything else handled by the browser is local */
    /* Anything not handled by the browser should be
     * passed to the relevant application, which in
     * turn gets to worry about connectedness on its own.
     */
    return TRUE;
}

static inline int64_t gettime_usec()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return (int64_t)tv.tv_sec * 1000000 + tv.tv_usec;
}

#define LOADING_PROGRESS_MINIMAL_TIME_INTERVAL (1000000 / 2)

static gboolean
send_progress_signal(GMozillaEngine *self, guint state, const gchar *url, gint cur, gint max)
{
  g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
  switch (state) {
      case LOADING_PROGRESS:
      {
//        if (self->progress_bar_state != 0)
        static int64_t previous_signal_time_usec = 0;
        int64_t current_time_usec = gettime_usec();
        self->progress_bar_state = 1;
        if (current_time_usec - previous_signal_time_usec < LOADING_PROGRESS_MINIMAL_TIME_INTERVAL)
           break;
        previous_signal_time_usec = current_time_usec;
        g_object_set_data(G_OBJECT(self), "async", (void*)1);
        g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[LOADING_PROGRESS] , 0, cur, max);
        break;
      }
      case START_LOADING:
      {
        if (self->progress_bar_state != 0) {
          if (url) {
            g_object_set_data(G_OBJECT(self), "async", (void*)1);
            g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[START_LOADING] , 0, url);
          }
        }
        self->progress_bar_state = 0;
        break;
      }
      case FINISHED_LOADING:
      {
        if (self->progress_bar_state != -1) {
          if (self->progress_bar_state == 0)
             send_progress_signal(self, LOADING_PROGRESS, NULL, 98, 99);
          g_object_set_data(G_OBJECT(self), "async", (void*)1);
          g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[FINISHED_LOADING], 0, self->last_net_status, self->page_size, FALSE);
        }
        self->progress_bar_state = -1;
        break;
      }
      default:
        break;
    }
    return TRUE;
}

static void 
move_focus_to_engine(GMozillaEngine *self)
{
    TRACE_LOG();
    if (self && GTK_IS_MOZ_EMBED(self->engine))
    {
        GtkWidget *child = gtk_bin_get_child(GTK_BIN(self->engine));
        if (child) 
        {
            gtk_widget_grab_focus(GTK_WIDGET(child));
        }
    }
}

static void
g_mozilla_engine_load_url (GMozillaEngine *self, const gchar *url)
{
    TRACE_LOG("url:%s", url);
    if (self && self->engine && GTK_IS_MOZ_EMBED(self->engine) && url)
    {
        user_iteraction_happens(self);
        EAL_IF_GFREE(self->will_open_location);
        self->page_size = 0;
        EAL_IF_GFREE(self->user_ctx->ctx_url);
        self->will_open_location = g_strdup(url);
        send_progress_signal(self, START_LOADING, url, 0, 0);
        ULOG_INFO_F("Engine: %p: LoadUrl: '%s'\n", self->engine, url);
        g_mozilla_web_force_online();
        if (self->engine)
            gtk_moz_embed_load_url(GTK_MOZ_EMBED(self->engine), url);
    }
}

static GWebStatus
g_mozilla_engine_load_follow_link (GMozillaEngine *self, const gchar *url)
{
    TRACE_LOG();
    /* MTBD | Not used */
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_load_url_with_css (GMozillaEngine *self, const gchar *url, const gchar *cssfile)
{
    TRACE_LOG();
    /* MTBD | Not Used */
    return G_WEB_STATUS_OK;
}

static void
g_mozilla_engine_load_url_with_js (GMozillaEngine *self, gboolean  enable)
{
    TRACE_LOG();
    /* MTBD | Not Used But can be usefull in future ;) */
}

static void
g_mozilla_engine_load_home_page (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    gchar * home_page = NULL;
    if (g_mozilla_engine_get_pref(G_TYPE_STRING, G_MOZILLA_PREF_HOME_PAGE_LOCATION, &home_page))
        if (home_page)
            g_mozilla_engine_load_url (self, home_page);
    g_mozilla_engine_notify_memory(self);
}

static void
g_mozilla_engine_stop_load (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    self->waiting_connection = FALSE;
    gtk_moz_embed_stop_load(GTK_MOZ_EMBED(self->engine));
    load_finished_cb(GTK_MOZ_EMBED(self->engine), self);
}

static void
g_mozilla_engine_reload (GMozillaEngine *self)
{
    TRACE_LOG();
    /*
       GTK_MOZ_EMBED_FLAG_RELOADNORMAL = 0;
       GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE = nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE;
       GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXY = nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY;
       GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE = (nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY | nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE);
       GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE = nsIWebNavigation::LOAD_FLAGS_CHARSET_CHANGE;
       */
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    user_iteraction_happens(self);
    gtk_moz_embed_reload(GTK_MOZ_EMBED(self->engine), GTK_MOZ_EMBED_FLAG_RELOADNORMAL);
    move_focus_to_engine(self);
}

static gboolean
navigation_idle_cb (gpointer data)
{
    TRACE_LOG();
    GMozillaEngine *self = G_MOZILLA_ENGINE(data);
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    
    user_iteraction_happens(self);
    if (self->nsteps == 1) {
        if (self->is_back)
            gtk_moz_embed_go_back(GTK_MOZ_EMBED(self->engine));
        else
            gtk_moz_embed_go_forward(GTK_MOZ_EMBED(self->engine));
    } else {
        gint cur_index = g_mozilla_engine_get_shistory_index (GTK_MOZ_EMBED(self->engine));
        gint index = 0;
        if (cur_index >= 0)
        {
            if (self->is_back)
                index = cur_index - self->nsteps;
            else
                index = cur_index + self->nsteps;
            g_mozilla_engine_shistory_goto_index(GTK_MOZ_EMBED(self->engine), index);
        }
    }
    move_focus_to_engine(self);
    return FALSE;
}

void
g_mozilla_engine_go_back (GMozillaEngine *self, guint nsteps)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    self->nsteps = nsteps;
    self->is_back = TRUE;
    self->navigation_event_id = g_idle_add((GSourceFunc)navigation_idle_cb, self);
}

void
g_mozilla_engine_go_forward (GMozillaEngine *self, guint nsteps)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    self->nsteps = nsteps;
    self->is_back = FALSE;
    self->navigation_event_id = g_idle_add((GSourceFunc)navigation_idle_cb, self);
}

static gboolean
g_mozilla_engine_can_go_back (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    return gtk_moz_embed_can_go_back(GTK_MOZ_EMBED(self->engine));
}

static gboolean
g_mozilla_engine_can_go_forward (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    return gtk_moz_embed_can_go_forward(GTK_MOZ_EMBED(self->engine));
}

static gboolean
g_mozilla_engine_set_zoom_level_wrap(GMozillaEngine *self, gint level, gpointer context)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);

#ifndef MOZ_CAIRO_GFX
    g_mozilla_engine_observe (G_MOZILLA_EXTENSIONS_IMAGEZOOM_CONTRACTID,
                              context, G_MOZILLA_SET_IMAGES_SCALING,
                              level == 100 ? NULL : (gunichar2*)"Do");
#endif
    gtk_moz_embed_set_zoom_level (GTK_MOZ_EMBED (self->engine), level, context);
#ifndef MOZ_CAIRO_GFX
    g_mozilla_engine_observe (G_MOZILLA_EXTENSIONS_IMAGEZOOM_CONTRACTID,
                              gtk_microb_context_get_node((GtkMicroBContext*)self->ctx),
                              G_MOZILLA_SCROLL_TO_LAST_NODE, NULL);
#endif
    if (self->optimized_view)
      g_mozilla_engine_observe (G_MOZILLA_EXTENSIONS_FTV_CONTRACTID, NULL,
                                G_MOZILLA_OPTIMIZED_VIEW, NULL);
    return TRUE;
}

static void
g_mozilla_engine_set_zoom_level (GMozillaEngine *self, gint level)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    if (level != self->current_zoom)
        g_mozilla_engine_set_zoom_level_wrap(self, level, NULL);
    self->current_zoom = level;
}

static GWebStatus
g_mozilla_engine_set_zoom_level_around_point (GMozillaEngine *self, guint x, guint y, gint level)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), G_WEB_ERR);
    if (!g_mozilla_engine_set_zoom_level_wrap(self, level, gtk_microb_context_get_node((GtkMicroBContext*)self->ctx)))
        return G_WEB_ERR;
    self->current_zoom = level;
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_compare_frames_zoom_level (GMozillaEngine *self, gboolean *equal)
{
    TRACE_LOG();
    //    gint zoom_level;
    //    gboolean tequal = TRUE;
    //  if (gtk_moz_embed_get_zoom_level(GTK_MOZ_EMBED(self->engine), &zoom_level, NULL, &tequal))
    if (equal) {
        *equal = TRUE;
    }
    //  else
    //    return G_WEB_ERR;
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_get_image_properties (GMozillaEngine *self, const gchar *url, const gchar **file_type, guint *file_size, guint *width, guint *height)
{
    TRACE_LOG("Url: '%s'", url);
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine && url
        && file_type && file_size && width && height, G_WEB_ERR);
    gtk_moz_embed_get_doc_info(GTK_MOZ_EMBED(self->engine),
                               gtk_microb_context_get_node((GtkMicroBContext*)self->ctx),
                               -1, &url, file_type, file_size, (gint*)width, (gint*)height);
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_get_frame_properties (GMozillaEngine *self, const gchar **file_type, guint *file_size, GWebEngineEncoding *file_encoding, guint *security, gboolean *automatic_encoding)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine && file_type
        && file_encoding && automatic_encoding, G_WEB_ERR);
    /* TBD */
    *automatic_encoding = TRUE;

    const gchar *url = self->user_ctx->ctx_docurl;
    if (self->doc_index == -1)
       url = gtk_moz_embed_get_location(GTK_MOZ_EMBED(self->engine));

    if (security)
        *security = self->sec_mode;

    gtk_moz_embed_get_doc_info(GTK_MOZ_EMBED(self->engine), NULL, -1, (const gchar**)&url, file_type, file_size, NULL, NULL);

    if (self->doc_index == -1)
      g_free((char*)url);
    
    if (file_size)
        if (*file_size == 0)
            *file_size = self->page_size;
    return G_WEB_STATUS_OK;
}

static GWebStatus
g_mozilla_engine_load_image (GMozillaEngine *self, const gchar* url)
{
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && url, G_WEB_ERR);
    TRACE_LOG("URL:%s\n", url);
    if (!g_mozilla_load_image(gtk_microb_context_get_node((GtkMicroBContext*)self->ctx), url))
        return G_WEB_ERR;
    
    return G_WEB_STATUS_OK;
}

static gint
g_mozilla_engine_get_back_window_history_list (GMozillaEngine *self,
                                               GObject **history)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && history, FALSE);
    *history = G_OBJECT(g_object_new(g_mozilla_history_list_get_type(), NULL));
    g_object_set_data(*history, "window-history", GINT_TO_POINTER(1));
    G_MOZILLA_HISTORY_LIST(*history)->isSHistory = TRUE;
    G_MOZILLA_HISTORY_LIST(*history)->num_items = g_mozilla_engine_get_shistory_list(self,
                                &(G_MOZILLA_HISTORY_LIST(*history)->items),
                                GTK_MOZ_COMMON_BACK_SHISTORY);
    return G_MOZILLA_HISTORY_LIST(*history)->num_items;
}

static gint
g_mozilla_engine_get_forward_window_history_list (GMozillaEngine *self,
                                                  GObject **history)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && history, FALSE);
    *history = G_OBJECT(g_object_new(g_mozilla_history_list_get_type(), NULL));
    g_object_set_data(*history, "window-history", GINT_TO_POINTER(1));
    G_MOZILLA_HISTORY_LIST(*history)->isSHistory = TRUE;
    G_MOZILLA_HISTORY_LIST(*history)->num_items = g_mozilla_engine_get_shistory_list(self,
                                &(G_MOZILLA_HISTORY_LIST(*history)->items),
                                GTK_MOZ_COMMON_FORWARD_SHISTORY);
    return G_MOZILLA_HISTORY_LIST(*history)->num_items;
}

guint
g_mozilla_engine_get_zoom_level (GMozillaEngine *self)
{
    TRACE_LOG();
    gint zoom = 100;
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, zoom);
    if (gtk_moz_embed_get_zoom_level(GTK_MOZ_EMBED(self->engine), &zoom, NULL))
        self->current_zoom = zoom;
    return self->current_zoom;
}

static void
g_mozilla_engine_zoom_by_steps (GMozillaEngine *self, guint relative_zoom, guint* legal_zoom_value)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_mozilla_engine_set_zoom_level_wrap(self, relative_zoom, NULL);
}

static void
g_mozilla_engine_new_zoom_by_steps (GMozillaEngine *self, gboolean up, guint* legal_zoom_value)
{
    TRACE_LOG();
    //gtk_moz_embed_zoom_by_steps(GTK_MOZ_EMBED(self->engine), up, legal_zoom_value);
}

static void
g_mozilla_engine_set_frame_zoom_level (GMozillaEngine *self, guint level)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_mozilla_engine_set_zoom_level_wrap(self, level, gtk_microb_context_get_node((GtkMicroBContext*)self->ctx));
}

static guint
g_mozilla_engine_get_frame_zoom_level (GMozillaEngine *self)
{
    TRACE_LOG();
    gint zoom = 100;
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, zoom);
    if (gtk_moz_embed_get_zoom_level(GTK_MOZ_EMBED(self->engine), &zoom, gtk_microb_context_get_node((GtkMicroBContext*)self->ctx)))
        return zoom;
    return 100;
}

static void
g_mozilla_engine_set_layout_mode (GMozillaEngine *self, GWebEngineLayoutMode mode)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    gboolean value = FALSE;
    switch (mode)
    {
    case G_WEBENGINE_LAYOUT_NORMAL:
        value = FALSE;
        break;
    case G_WEBENGINE_LAYOUT_SSR:
        value = TRUE;
        break;
    case G_WEBENGINE_LAYOUT_MSR:
        value = TRUE;
        break;
    default:
        value = FALSE;
    }

    if (g_mozilla_engine_set_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_SSR_ENABLED, &value))
        g_mozilla_engine_save_prefs();

    if (self->optimized_view == value)
        return;

    self->optimized_view = value;
}

static GWebEngineLayoutMode
g_mozilla_engine_get_layout_mode (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), G_WEBENGINE_LAYOUT_NORMAL);
    if (g_mozilla_engine_get_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_SSR_ENABLED, &self->optimized_view))
        if (self->optimized_view)
            return G_WEBENGINE_LAYOUT_MSR;

    return G_WEBENGINE_LAYOUT_NORMAL;
}

static void
g_mozilla_engine_set_load_images(GMozillaEngine *self, GWebEngineImagePolicy level)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_mozilla_set_image_policy(self, level);
}

static GWebEngineImagePolicy
g_mozilla_engine_get_load_images (GMozillaEngine *self)
{
    TRACE_LOG();
    gint show_images = 1;
    if (!g_mozilla_engine_get_pref(G_TYPE_INT, G_MOZILLA_PREF_SHOW_IMAGES, &show_images))
        return G_WEBENGINE_POLICY_ALL_IMAGES;

    gboolean download_images = TRUE;
    switch (show_images)
    {
    case 1:
        // NOTE: G_MOZILLA_PREF_DOWNLOAD_IMAGES="microb.download_images", 
        //       see: microbImageLoadingPolicy.js
        if (g_mozilla_engine_get_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_DOWNLOAD_IMAGES, &download_images))
            if (!download_images)
                return G_WEBENGINE_POLICY_LOADED_IMAGES;
        return G_WEBENGINE_POLICY_ALL_IMAGES;
    case 2:
        return G_WEBENGINE_POLICY_NO_IMAGES;
    default:
        return G_WEBENGINE_POLICY_ALL_IMAGES;
    }
}

static void
g_mozilla_engine_set_frame_stacking (GMozillaEngine *self, gboolean enabled)
{
    TRACE_LOG();
}

static gboolean
g_mozilla_engine_get_frame_stacking (GMozillaEngine *self)
{
    TRACE_LOG();
    gboolean fr_stacking;

    return (g_mozilla_engine_get_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_MERGE_FRAMES, &fr_stacking) ? fr_stacking : FALSE);
}

static GWebStatus
g_mozilla_engine_save_web_page (GMozillaEngine *self, GWebEngineSaveAction setting, const gchar *file_name_with_path)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine
        && file_name_with_path, G_WEB_ERR);
    gchar * url = gtk_moz_embed_get_location(GTK_MOZ_EMBED(self->engine));
    GWebStatus status = G_WEB_ERR;
    //FIXME g_mozilla_engine_save_target
    if (g_mozilla_engine_save_target (self, url, (gchar*)file_name_with_path, setting, NULL))
        status = G_WEB_STATUS_OK;
    g_free(url);
    return status;
}

static GWebStatus
g_mozilla_engine_save_current_frame (GMozillaEngine *self, GWebEngineSaveAction setting, const gchar *file_name_with_path)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine
        && file_name_with_path, G_WEB_ERR);
    gchar * url = gtk_moz_embed_get_location(GTK_MOZ_EMBED(self->engine));
    GWebStatus status = G_WEB_ERR;
    if (g_mozilla_engine_save_target (self, url, (gchar*)file_name_with_path, setting, gtk_microb_context_get_node((GtkMicroBContext*)self->ctx)))
        status = G_WEB_STATUS_OK;
    g_free(url);
    return status;
}

static const gchar*
g_mozilla_engine_get_frame_title (GMozillaEngine *self, gint frame_number)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), NULL);
    // bug 46234 - solved
    const gchar * frame_title = g_value_get_string(gtk_microb_context_get_prop((GtkMicroBContext*)self->ctx, "doctitle"));
    return frame_title;
}

static gchar*
g_mozilla_engine_get_element_details (GMozillaEngine *self, gint type)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), NULL);
    gchar *element_details = NULL;
    element_details = g_value_dup_string(gtk_microb_context_get_prop((GtkMicroBContext*)self->ctx, "title"));
    return element_details;
}

static gint
g_mozilla_engine_get_frame_number (GMozillaEngine *self)
{
    TRACE_LOG();
    gint frame_number = 0;
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), frame_number);
    frame_number = self->doc_index;
    /*IMP
      frame_number = gtk_moz_embed_get_frame_number(GTK_MOZ_EMBED(self->engine), NULL[Title], NULL[Location]);
      */
    return frame_number;
}

static GWebStatus
g_mozilla_engine_save_web_page_image (GMozillaEngine *self, const gchar *url, const gchar* path)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), G_WEB_ERR);
    //FIXME g_mozilla_engine_save_target
    if (!g_mozilla_engine_save_target (self, (gchar*)url, (gchar*)path, 2, NULL))
        return G_WEB_ERR;
    return G_WEB_STATUS_OK;
}

GWebEngineEncoding
g_mozilla_engine_get_encoding (GtkMozEmbed *embed)
{
    gchar *encoding;

    encoding = g_moz_engine_get_encoding (embed, -1);

    if (!encoding)
        return 0;

    if (g_ascii_strcasecmp(encoding, "UTF-8") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_UTF8; };
    if (g_ascii_strcasecmp(encoding, "UTF-16BE") == 0 ||
        g_ascii_strcasecmp(encoding, "UTF-16LE") == 0 ||
        g_ascii_strcasecmp(encoding, "UTF-16") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_UTF16; };
    if (g_ascii_strcasecmp(encoding, "ISO-8859-1") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO8859_1; };
    if (g_ascii_strcasecmp(encoding, "ISO-8859-15") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO8859_15; };
    if (g_ascii_strcasecmp(encoding, "ISO-8859-2") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO8859_2; };
    if (g_ascii_strcasecmp(encoding, "ISO-8859-7") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO8859_7; };
    if (g_ascii_strcasecmp(encoding, "ISO-8859-9") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO8859_9; };
    if (g_ascii_strcasecmp(encoding, "windows-1250") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_CP_1250; };
    if (g_ascii_strcasecmp(encoding, "windows-1251") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_CP_1251; };
    if (g_ascii_strcasecmp(encoding, "windows-1253") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_CP_1253; };
    if (g_ascii_strcasecmp(encoding, "windows-1254") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_CP_1254; };
    if (g_ascii_strcasecmp(encoding, "KOI8-R") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_KOI8_R; };
    if (g_ascii_strcasecmp(encoding, "BIG5") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_BIG5; };
    if (g_ascii_strcasecmp(encoding, "GB18030") == 0 ||
        g_ascii_strcasecmp(encoding, "GBK") == 0 ||
        g_ascii_strcasecmp(encoding, "X-GBK") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_GB_18030; };
    if (g_ascii_strcasecmp(encoding, "ISO-2022-CN") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO_2022_CN; };
    if (g_ascii_strcasecmp(encoding, "EUC-TW") == 0 ||
        g_ascii_strcasecmp(encoding, "X-EUC-TW") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_EUC_TW; };
    if (g_ascii_strcasecmp(encoding, "windows-1256") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_CP_1256; };
    if (g_ascii_strcasecmp(encoding, "GB2312") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_GB2312; };
    if (g_ascii_strcasecmp(encoding, "HZ") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_HZ_GB2312; };
    if (g_ascii_strcasecmp(encoding, "EUC-KR") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_EUC_KR; };
    if (g_ascii_strcasecmp(encoding, "TCVN") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_TCVN; };
    if (g_ascii_strcasecmp(encoding, "VPS") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_VPS; };
    if (g_ascii_strcasecmp(encoding, "VISCII") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_VISCII; };
    if (g_ascii_strcasecmp(encoding, "KOI8-U") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_KOI8_U; };
    if (g_ascii_strcasecmp(encoding, "ISO-8859-6") == 0) { g_free(encoding); return G_WEBENGINE_ENCODING_ISO8859_6; };

    /* we should never get here, but if we do we raise an error */
    return 0;
}

static void
g_mozilla_engine_set_encoding (GMozillaEngine *self, GWebEngineEncoding encoding, gint frame_number)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    gchar * encoding_text = NULL;
    switch (encoding)
    {
    case G_WEBENGINE_ENCODING_UTF8:
        encoding_text = "UTF-8";
        break;
    case G_WEBENGINE_ENCODING_UTF16:
        encoding_text = "UTF-16BE";
        break;
    case G_WEBENGINE_ENCODING_ISO8859_1:
        encoding_text = "ISO-8859-1";
        break;
    case G_WEBENGINE_ENCODING_ISO8859_15:
        encoding_text = "ISO-8859-15";
        break;
    case G_WEBENGINE_ENCODING_ISO8859_2:
        encoding_text = "ISO-8859-2";
        break;
    case G_WEBENGINE_ENCODING_ISO8859_7:
        encoding_text = "ISO-8859-7";
        break;
    case G_WEBENGINE_ENCODING_ISO8859_9:
        encoding_text = "ISO-8859-9";
        break;
    case G_WEBENGINE_ENCODING_CP_1250:
        encoding_text = "windows-1250";
        break;
    case G_WEBENGINE_ENCODING_CP_1251:
        encoding_text = "windows-1251";
        break;
    case G_WEBENGINE_ENCODING_CP_1253:
        encoding_text = "windows-1253";
        break;
    case G_WEBENGINE_ENCODING_CP_1254:
        encoding_text = "windows-1254";
        break;
    case G_WEBENGINE_ENCODING_KOI8_R:
        encoding_text = "KOI8-R";
        break;
    case G_WEBENGINE_ENCODING_BIG5:
        encoding_text = "BIG5";
        break;
    case G_WEBENGINE_ENCODING_GB_18030:
        encoding_text = "GB18030";
        break;
    case G_WEBENGINE_ENCODING_ISO_2022_CN:
        encoding_text = "ISO-2022-CN";
        break;
    case G_WEBENGINE_ENCODING_EUC_TW:
        encoding_text = "EUC-TW";
        break;
    case G_WEBENGINE_ENCODING_CP_1256:
        encoding_text = "windows-1256";
        break;
    case G_WEBENGINE_ENCODING_GB2312:
        encoding_text = "GB2312";
        break;
    case G_WEBENGINE_ENCODING_HZ_GB2312:
        encoding_text = "HZ";
        break;
    case G_WEBENGINE_ENCODING_GBK:
        encoding_text = "GBK";
        break;
    case G_WEBENGINE_ENCODING_EUC_KR:
        encoding_text = "EUC-KR";
        break;
    case G_WEBENGINE_ENCODING_TCVN:
        encoding_text = "TCVN";
        break;
    case G_WEBENGINE_ENCODING_VPS:
        encoding_text = "VPS";
        break;
    case G_WEBENGINE_ENCODING_VISCII:
        encoding_text = "VISCII";
        break;
    case G_WEBENGINE_ENCODING_KOI8_U:
        encoding_text = "KOI8-U";
        break;
    case G_WEBENGINE_ENCODING_ISO8859_6:
        encoding_text = "ISO-8859-6";
        break;
    default:
        return;
    }
    char * cur_encoding_text = g_moz_engine_get_encoding (GTK_MOZ_EMBED(self->engine), frame_number);
    g_return_if_fail(cur_encoding_text);
    if (0 != strcmp(cur_encoding_text, encoding_text)) {
        g_moz_engine_set_encoding(GTK_MOZ_EMBED(self->engine), encoding_text, frame_number);
        if (!self->is_loading)
            g_mozilla_engine_reload (self);
    }
    g_free(cur_encoding_text);
}

static void
g_mozilla_engine_set_language_encoding (GMozillaEngine *self, GWebEngineEncoding encoding)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_mozilla_engine_set_encoding (self, encoding, -1);
}

static void
g_mozilla_engine_set_frame_encoding (GMozillaEngine *self, GWebEngineEncoding encoding)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_mozilla_engine_set_encoding (self, encoding, self->doc_index); //0|FRAME_NUMBER
}

static gboolean
g_mozilla_engine_find_text (GMozillaEngine *self, const gchar *string, gboolean reverse, gboolean whole_word, gboolean case_sensitive, gboolean restart)
{
    TRACE_LOG();
    g_return_val_if_fail (G_MOZILLA_ENGINE (self) && self->engine, G_WEBENGINE_SEARCH_NOTFOUND);
    return (gboolean)g_mozilla_find_text (self, string, reverse,
                                          case_sensitive, restart, -1);
}

static GWebEngineSearchResult
g_mozilla_engine_type_find_text (GMozillaEngine *self, const gchar *string, gboolean reverse, gboolean case_sensitive, gboolean restart)
{
    TRACE_LOG();
    g_return_val_if_fail (G_MOZILLA_ENGINE (self) && self->engine, G_WEBENGINE_SEARCH_NOTFOUND);
    return (GWebEngineSearchResult)g_mozilla_find_text (self, string, reverse,
                                                        case_sensitive, restart, -1);
}

static void
g_mozilla_engine_authenticate (GMozillaEngine *self, gboolean ok, const gchar *username, const gchar *password)
{
    TRACE_LOG();

    if (ok && self)
    {
        EAL_IF_GFREE(self->username);
        EAL_IF_GFREE(self->password);

        if (username)
            self->username = g_strdup(username);
        if (password)
            self->password = g_strdup(password);
        self->accept = TRUE;
    }
}

static void
g_mozilla_engine_select_all_text (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    if (!g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_SELECT_ALL))
        ULOG_WARN_F("Failed to do GTK_MOZ_CLIPBOARD_SELECT_ALL");
}

static gboolean
g_mozilla_engine_can_selecttext (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    gboolean status = TRUE;
    if (ctx_text_input(self))
        status = gtk_microb_context_get_text((GtkMicroBContext*)self->ctx, NULL, NULL);
    return status;
}

static GWebStatus
g_mozilla_engine_cut (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, G_WEB_ERR);
    if (g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_CUT))
        return G_WEB_STATUS_OK;
    return G_WEB_ERR;
}

static GWebStatus
g_mozilla_engine_copy (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, G_WEB_ERR);
    if (g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_COPY))
        return G_WEB_STATUS_OK;
    return G_WEB_ERR;
}

static GWebStatus
g_mozilla_engine_paste (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, G_WEB_ERR);
    if (g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_PASTE))
        return G_WEB_STATUS_OK;
    return G_WEB_ERR;
}

static gboolean
g_mozilla_engine_can_cut (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    if (!g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_CAN_CUT) ||
        (self->user_ctx->common_data.ctx_type == G_WEBENGINE_POPUPMENU_NOTYPE))
        return FALSE;
    return TRUE;
}

static gboolean
g_mozilla_engine_can_paste (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    if (!g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_CAN_PASTE))
        return FALSE;
    return TRUE;
}

static gboolean
g_mozilla_engine_can_copy (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    //Disabled copy option in password fields.
    if (!g_mozilla_engine_clipboard(GTK_MOZ_EMBED(self->engine), GTK_MOZ_CLIPBOARD_CAN_COPY) ||
        ctx_password(self))
        return FALSE;
    return TRUE;
}

static gchar*
g_mozilla_engine_get_favicon_name (GMozillaEngine *self, const gchar *favicon_folder)
{
    TRACE_LOG();
    /* TBD | It is really usefull stuff for us? I think favicon_signal is enough */
    /*
       If yes, then we should save favicon for current url into favicon_folder, and provide path for it
       */
    return NULL;
}

static GObject*
g_mozilla_engine_get_engine_widget (GMozillaEngine *self)
{
    TRACE_LOG();
    return self->notification_vbox;
}

static GObject*
g_mozilla_engine_get_engine_notifier (GMozillaEngine *self)
{
    TRACE_LOG("Notifier: %p", self);
    return G_OBJECT(self);
}

static void
g_mozilla_engine_set_open_link_mode (GMozillaEngine *self, gboolean in_new_window)
{
    TRACE_LOG();
    /* MTBD | Not used*/
}

static GWebStatus
g_mozilla_engine_set_true_zoom_mode (GMozillaEngine *self, gboolean status)
{
    TRACE_LOG();
    /* MTBD | Don't know what it's mean :( */
    return G_WEB_STATUS_OK;
}

static void
g_mozilla_engine_notify_memory (GMozillaEngine *self)
{
    g_mozilla_engine_observe(NULL, NULL, G_MOZILLA_MEMORY_PRESSURE, (gunichar2 *)G_MOZILLA_HEAP_MINIMIZE_UNICODE);
    malloc_trim(0);
}

static void
g_mozilla_engine_notify_plugins (GMozillaEngine *self, gint event)
{
    TRACE_LOG();
    if (self && self->engine)
      gtk_moz_embed_notify_plugins(GTK_MOZ_EMBED(self->engine), event);
    if (G_WEB_PLUGINS_LOW_MEMORY != event)
      return;
    g_mozilla_engine_notify_memory(self);
}

void
g_mozilla_engine_update_area (GMozillaEngine *self, int id, int left, int top, int width, int height, int zoom)
{
    TRACE_LOG();
    draw_layout(self, id, left, top, width, height, zoom);
}

void
g_mozilla_engine_visible_area (GMozillaEngine *self, int vx, int vy, int vw, int vh,
                               int tx, int ty, int tw, int th, int wx, int wy, int ww, int wh)
{
    TRACE_LOG();
    g_return_if_fail(self);

    ULOG("vis:[%i,%i,%i,%i], tr[%i,%i,%i,%i], window[%i,%i,%i,%i]",
         vx, vy, vw, vh, tx, ty, tw, th, wx, wy, ww, wh);

    self->mLastVisibleArea.x = vx;
    self->mLastVisibleArea.y = vy;
    self->mLastVisibleArea.width = vw;
    self->mLastVisibleArea.height = vh;

    self->mLastTargetArea.x = tx;
    self->mLastTargetArea.y = ty;
    self->mLastTargetArea.width = tw;
    self->mLastTargetArea.height = th;

    if ((ww && self->mWindowSize.width != ww)
       || (wh && self->mWindowSize.height != wh)
       || (self->mWindowSize.x != wx)
       || (self->mWindowSize.y != wy)
    ) {
      self->mPrevScrollPos.x = self->mWindowSize.x;
      self->mPrevScrollPos.y = self->mWindowSize.y;
      self->mWindowSize.x = wx;
      self->mWindowSize.y = wy;
      self->mWindowSize.width = ww;
      self->mWindowSize.height = wh;
      update_default_window_size(self);
    }
}

static void g_mozilla_engine_update_context_urls (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));

    const GValue *val = NULL;
    
    EAL_IF_GFREE(self->user_ctx->ctx_url);
    val = gtk_microb_context_get_prop((GtkMicroBContext*)self->ctx, "link");
    if (val)
        self->user_ctx->ctx_url = g_value_dup_string(val);

    EAL_IF_GFREE(self->user_ctx->ctx_objurl);
    val = gtk_microb_context_get_prop((GtkMicroBContext*)self->ctx, "image");
    if (val)
        self->user_ctx->ctx_objurl = g_value_dup_string(val);

    EAL_IF_GFREE(self->user_ctx->ctx_docurl);
    val = gtk_microb_context_get_prop((GtkMicroBContext*)self->ctx, "docuri");
    if (val)
        self->user_ctx->ctx_docurl = g_value_dup_string(val);
}

void
g_mozilla_engine_send_mouse_event (GMozillaEngine *self, int type, int x, int y, int mod, int button, int clickcount, int time)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));

    if (type == G_WEBENGINE_UI_WINDOW_INFO) {
      ULOG("UI_XWINDOWID:%lu, widget:%p", (long unsigned int)mod, self->engine);
      self->ui_xwin_id = mod;
      g_object_set_data(self->engine, "UI_XID", GINT_TO_POINTER(self->ui_xwin_id));
      return;
    }

    if (type == G_WEBENGINE_FOCUS_CHANGE) {
      ULOG("FOCUS_CHANGE:%lu, widget:%p", (long unsigned int)mod, self->engine);
      if (mod && ctx_text_input_or_password(self))
        gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);

      if (!self->is_vkb_showed)
        g_mozilla_engine_send_activate_event(self, mod);
      return;
    }

    if (type == G_WEBENGINE_ACTIVE_STATUS) {
      if (clickcount) {
        if (!G_MOZILLA_WEB(self->global)->mHideFixedFrames) return;
        ULOG("%s fixed frames", mod ? "Enable" : "Disable");
        self->mPaintFixedFrames = mod ? TRUE : FALSE;
        g_mozilla_engine_update_fixed_frames(self, FALSE);
        return;
      }
      ULOG_INFO("SET_ACTIVE:%p, %i->%i, is_layout_active:%i, ChangeSusp:%s", self, self->is_active, mod, self->is_layout_active, button ? "TRUE" : "FALSE");
      if (self->is_active != mod)
        g_mozilla_xshm_buffer_set_block(self, FALSE);
      self->is_active = mod;

      g_mozilla_engine_send_activate_event(self, mod);

      if (!button || self->is_layout_active == mod)
        return;

      g_mozilla_engine_window_set_active(self, mod);
      return;
    }

    move_focus_to_engine(self);

    if (type == G_WEBENGINE_MOUSE_SCROLL) 
    {
      int overflowx=0, overflowy=0;
      scroll_moz_scrollable_view(self, x, y, &overflowx, &overflowy);
      if (overflowx || overflowy)
        send_frame_scroll_overflow_message (self, overflowx, overflowy);
      return;
    }

    if (type == G_WEBENGINE_ELEMENT_INFO) {
      GdkRectangle rect;
      if (x >= 0 || y >= 0)
        get_element_info(self, x, y, time, &rect);
      else
        self->recalculate_element = TRUE;
      return;
    }

    if (type == G_WEBENGINE_MOUSE_CLICK || type == G_WEBENGINE_MOUSE_DOWN ||
        type == G_WEBENGINE_MOUSE_UP)
    {
        self->dom_focused_input = FALSE;
        self->menu_shown = FALSE;
        // Resolve the element underneath the mouse cursor. If the element is
        // of type 'select', do not send the event to the layout. Otherwise,
        // a native selection menu is displayed and we want to show a custom
        // widget instead.
        GdkRectangle rect;
        get_element_info(self, x, y, 0, &rect);
        g_mozilla_engine_update_context_data(self, CTX_USER, GTK_MOZ_EMBED(self->engine), NULL);
        if (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_SELECT)
        {
            if (type != G_WEBENGINE_MOUSE_DOWN)
                create_select_widget (self,
                                      gtk_microb_context_get_node((GtkMicroBContext*)self->ctx),
                                      gtk_microb_context_get_original_node((GtkMicroBContext*)self->ctx));

            return;
        }
    }

    if (type == G_WEBENGINE_MOUSE_LONG_PRESS) {
      self->menu_shown = FALSE;
      self->mouse_down = TRUE;
    }

    if (type != G_WEBENGINE_MOUSE_DOWN &&
        (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT ||
         self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD))
    {
        EAL_IF_GFREE_FUNC(self->fake_key_id, g_source_remove);
        send_fake_key_event_to_layout(self);
    }

    send_event_to_layout(self, type, x, y, mod, button, clickcount, time);
}

static void
g_mozilla_engine_preload_current_skin (GMozillaEngine *self)
{
    TRACE_LOG();
}

static void
g_mozilla_engine_invoke_selectmatch (GMozillaEngine *self)
{
    TRACE_LOG();
}

GArray *
g_mozilla_engine_get_server_certificate (GMozillaEngine *self, gpointer context)
{
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), NULL);
    gpointer severCert = NULL;
    gboolean status = FALSE;
    guint len = 0;
    unsigned char *p = NULL;

    if (!context) {
      status = g_mozilla_get_server_cert(GTK_MOZ_EMBED(self->engine), &severCert, NULL);
      if (!status || severCert == NULL)
          return NULL;
    }

    p = gtk_moz_embed_common_nsx509_to_raw(context?((GMozillaWebCertContext*)context)->mozcert:severCert, &len);

    GArray *res = g_array_new(TRUE, TRUE, len);
    int i = 0;
    for (i = 0; i<len; i++)
      g_array_append_val(res, p[i]);

    return res;
}

gchar*
g_mozilla_engine_get_page_verifier_name (GMozillaEngine *self)
{
    gchar* verifier_name = NULL;
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), verifier_name);

    g_mozilla_get_page_verifier_name(GTK_MOZ_EMBED(self->engine), &verifier_name);

    return verifier_name;
}

#include <unistd.h>
gboolean
g_mozilla_engine_permit_page_unload (GMozillaEngine *self)
{
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    gboolean ret = g_moz_engine_permit_unload (GTK_MOZ_EMBED(self->engine));
    return ret;
}

static void
g_mozilla_engine_destroy_updates_list(GMozillaEngine *self)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    if (!self->mBlockedUIRequests) return;

    GList * ptr = (GList*)self->mBlockedUIRequests;
    while(ptr)
    {
        EAL_IF_GFREE(ptr->data);
        ptr = ptr->next;
    }
    EAL_IF_GFREE_FUNC(self->mBlockedUIRequests, g_list_free);
}

static void
g_mozilla_reset_context_info(GMozillaEngine *self)
{
    self->last_event = NULL;
    g_object_unref(self->ctx);
}

static void
g_mozilla_engine_reset (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));

    ULOG("reset:e:%p, g:%p, l:%s", self, self->engine, self->last_location);

    G_MOZILLA_WEB(self->global)->window_list = g_list_remove_all (G_MOZILLA_WEB(self->global)->window_list, self);
    // Make sure that we are not blocking buffer

    g_mozilla_unregister_im_context(self);

    if (self->is_active)
      g_mozilla_xshm_buffer_set_block(self, FALSE);

    g_mozilla_engine_signals_disconnect(self);

    EAL_IF_GFREE_FUNC(self->mBlockedUpdatesSource, g_source_remove);
    EAL_IF_GFREE_FUNC(self->mBlockedUpdates, gdk_region_destroy);
    EAL_IF_GFREE_FUNC(self->scroll_to_node_timer_id, g_source_remove);
    EAL_IF_GFREE_FUNC(self->suspend_timeouts_timer_id, g_source_remove);
    EAL_IF_GFREE_FUNC(self->fake_key_id, g_source_remove);
    EAL_IF_GFREE_FUNC(self->navigation_event_id, g_source_remove);

    EAL_IF_GFREE_FUNC(self->mBlockedUIRequestsSource, g_source_remove);
    EAL_IF_GFREE_FUNC(self->mForcedUpdateTimeout, g_source_remove);
    EAL_GFREE(self->prev_find_str);
    g_mozilla_engine_destroy_updates_list(self);

    if (self->prev_eventkey)
    {
        gdk_event_free((GdkEvent*)self->prev_eventkey);
        self->prev_eventkey = NULL;
    }

    g_mozilla_reset_context_info(self);
    g_mozilla_engine_destroy_internal(self);

    if (!self->engine)
      return;

    g_object_steal_data(G_OBJECT(self->engine), G_MOZILLA_EAL_PARENT);
    GtkWidget * mozCont = g_mozilla_engine_get_mozcontainer(self);
    if (mozCont)
        g_object_steal_data(G_OBJECT(mozCont), G_MOZILLA_EAL_PARENT);
    g_object_steal_data(G_OBJECT(self->engine), G_MOZILLA_EAL_CTX_TYPE);

    GtkWidget* widget_ptr = (GTK_WIDGET(self->engine));

    EAL_IF_GFREE_FUNC(widget_ptr, gtk_widget_destroy);
}

static gboolean
g_mozilla_engine_realize_window (GMozillaEngine *self)
{
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    g_return_val_if_fail(GTK_IS_MOZ_EMBED(self->engine), FALSE);
    gtk_moz_embed_init_window(GTK_MOZ_EMBED(self->engine));
    GtkWidget * mozCont = g_mozilla_engine_get_mozcontainer(self);
    if (mozCont)
        g_object_set_data(G_OBJECT(mozCont), G_MOZILLA_EAL_PARENT, self);
    return TRUE;
}

static gboolean 
g_mozilla_engine_get_thumbnail(GMozillaEngine *self, const gchar* thumbnail_filename, gint width, gint height)
{
    return FALSE;
}

static gboolean
g_mozilla_engine_evaluate_js(GMozillaEngine *self, const gchar* js)
{
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);

    if (!g_mozilla_evaluate_js(self, js))
        return TRUE;

    return FALSE;
}

static void
g_mozilla_engine_enable_shistory_thumbnails(GMozillaEngine *self, gboolean enable, gint width, gint height)
{
}

gpointer g_mozilla_append_notification(GMozillaEngine* self,
                                       const gchar *notification_label,
                                       gpointer notification)
{
    // Create the notification bar
    GtkWidget *notification_bar = GTK_WIDGET(gtk_vbox_new(FALSE, 0));

    // Create the label
    GtkWidget* label_widget = GTK_WIDGET(gtk_label_new(notification_label));
    g_object_set(G_OBJECT(label_widget), "wrap", TRUE, "wrap-mode", PANGO_WRAP_WORD, NULL);
    gtk_widget_set_size_request(label_widget, MAXIMUM_DIALOG_WIDTH, -1);
    
    // Add the label to top of the notification 
    gtk_box_pack_start(GTK_BOX(notification_bar), label_widget, FALSE, FALSE, 1);

    return notification_bar;
}

void g_mozilla_add_notification_button(GMozillaEngine* self,
                gpointer notification_widget,
                gpointer notification,
                gpointer notification_button,
                gchar *label)
{
    if ((NULL == notification_widget)
      || (NULL == notification)
      || (NULL == notification_button)
      || (NULL == label))
    {
        return;
    }

    GtkWidget *notification_bar = GTK_WIDGET(notification_widget);
    HildonSizeType size = HILDON_SIZE_HALFSCREEN_WIDTH | HILDON_SIZE_FINGER_HEIGHT;
    HildonButtonArrangement arrangement = HILDON_BUTTON_ARRANGEMENT_HORIZONTAL;
    GtkWidget* button = hildon_button_new_with_text(size, arrangement, label, NULL);
    g_object_set_data(G_OBJECT(button), "button-data", notification_button);
    gtk_container_add(GTK_CONTAINER(notification_bar), button);
    g_signal_connect(G_OBJECT(button),
                     "clicked",
                     G_CALLBACK(g_mozilla_notification_button_pressed),
                     notification);
}

static gboolean notification_plug_deleted_cb(GtkWidget *widget,
                                             GdkEvent *event,
                                             GMozillaEngine *self)
{
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    self->dialog_is_active = FALSE;
    return FALSE;
}

void g_mozilla_send_notification(GMozillaEngine* self,
                                 gpointer notification_widget,
                                 const gchar *title)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    GtkWidget *plug = GTK_WIDGET(gtk_plug_new (0));
    g_signal_connect(plug, "delete-event", (GCallback) notification_plug_deleted_cb, self);
    gtk_container_add (GTK_CONTAINER (plug), GTK_WIDGET (notification_widget));
    gtk_widget_show_all (GTK_WIDGET (plug));
    
    guint32 xid = (guint32) gtk_plug_get_id (GTK_PLUG(plug));
    g_signal_emit_by_name (G_OBJECT(self), G_WEBWIDGET_SIGNAL_NOTIFICATION_DIALOG, xid, title);
}

void g_mozilla_show_notification(GMozillaEngine* self,
                gpointer notification_widget,
                gpointer notification,
                const char *title)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    
    if ((NULL == notification_widget) || (NULL == notification))
    {
        return;
    }

    GtkWidget *notification_bar = GTK_WIDGET(notification_widget);
    gtk_widget_show_all(notification_bar);
    g_mozilla_send_notification (self, notification_bar, title ? title : _(TR_WEB_REMEMBER_PASSWORD));
}

void g_mozilla_remove_notification(GMozillaEngine* self,
                gpointer notification_widget)
{
    if (NULL == notification_widget)
        return;

    GtkWidget *parent = gtk_widget_get_parent(GTK_WIDGET(notification_widget));
    gtk_widget_destroy(GTK_WIDGET(notification_widget));
    if (parent) {
        gtk_widget_destroy(GTK_WIDGET(parent));
        self->dialog_is_active = FALSE;
    }
}


static gint
g_mozilla_engine_get_text_zoom_level (GMozillaEngine *self)
{
    TRACE_LOG();
    gint zoom_level = 100;
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, zoom_level);

    zoom_level = g_mozilla_get_text_zoom(GTK_MOZ_EMBED(self->engine));
    if (zoom_level >= 0)
        self->current_zoom = zoom_level;
    else
        zoom_level = 100;

    return zoom_level;
}

static void
g_mozilla_engine_set_text_zoom_level (GMozillaEngine *self, gint level)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self) && self->engine);
    if (level != self->current_zoom)
        if (g_mozilla_set_text_zoom(GTK_MOZ_EMBED(self->engine), level))
            self->current_zoom = level;
}

static void
g_mozilla_engine_interface_init (GWebEngineIface *iface)
{
    TRACE_LOG();
    iface->destroy                         = (void (*) (GWebEngine*))
        g_mozilla_engine_destroy;
    iface->reset                           = (void (*) (GWebEngine*))
        g_mozilla_engine_reset;
    iface->load_url                        = (void (*) (GWebEngine*, const gchar*))
        g_mozilla_engine_load_url;
    iface->load_follow_link                = (GWebStatus (*) (GWebEngine*, const gchar*))
        g_mozilla_engine_load_follow_link;
    iface->load_url_with_css               = (GWebStatus (*) (GWebEngine*, const gchar*, const gchar*))
        g_mozilla_engine_load_url_with_css;
    iface->load_url_with_js                = (void (*) (GWebEngine*, gboolean))
        g_mozilla_engine_load_url_with_js;
    iface->load_home_page                  = (void (*) (GWebEngine*))
        g_mozilla_engine_load_home_page;
    iface->stop_load                       = (void (*) (GWebEngine*))
        g_mozilla_engine_stop_load;
    iface->reload                          = (void (*) (GWebEngine*))
        g_mozilla_engine_reload;
    iface->go_back                         = (void (*) (GWebEngine*, guint))
        g_mozilla_engine_go_back;
    iface->go_forward                      = (void (*) (GWebEngine*, guint))
        g_mozilla_engine_go_forward;
    iface->can_go_back                     = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_can_go_back;
    iface->can_go_forward                  = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_can_go_forward;
    iface->set_zoom_level                  = (void (*) (GWebEngine*, gint))
        g_mozilla_engine_set_zoom_level;
    iface->zoom_by_steps                   = (void (*) (GWebEngine*, guint, guint *))
        g_mozilla_engine_zoom_by_steps;
    iface->new_zoom_by_steps               = (void (*) (GWebEngine*, gboolean, guint *))
        g_mozilla_engine_new_zoom_by_steps;
    iface->set_zoom_level_around_point     = (GWebStatus (*) (GWebEngine*, guint, guint, gint))
        g_mozilla_engine_set_zoom_level_around_point;
    iface->compare_frames_zoom_level       = (GWebStatus (*) (GWebEngine*, gboolean*))
        g_mozilla_engine_compare_frames_zoom_level;
    iface->get_image_properties            = (GWebStatus (*) (GWebEngine*, const gchar*, gchar**, guint*, guint*, guint*))
        g_mozilla_engine_get_image_properties;
    iface->get_frame_properties            = (GWebStatus (*) (GWebEngine*, gchar**, guint*, GWebEngineEncoding*, guint*, gboolean*))
        g_mozilla_engine_get_frame_properties;
    iface->load_image                      = (GWebStatus (*) (GWebEngine*, const gchar*))
        g_mozilla_engine_load_image;
    iface->get_back_window_history_list    = (gint (*) (GWebEngine*, GWebHistoryList **))
        g_mozilla_engine_get_back_window_history_list;
    iface->get_forward_window_history_list = (gint (*) (GWebEngine*, GWebHistoryList **))
        g_mozilla_engine_get_forward_window_history_list;
    iface->get_zoom_level                  = (guint (*) (GWebEngine*))
        g_mozilla_engine_get_zoom_level;
    iface->set_frame_zoom_level            = (void (*) (GWebEngine*, guint))
        g_mozilla_engine_set_frame_zoom_level;
    iface->get_frame_zoom_level            = (guint (*) (GWebEngine*))
        g_mozilla_engine_get_frame_zoom_level;
    iface->set_layout_mode                 = (void (*) (GWebEngine*, GWebEngineLayoutMode))
        g_mozilla_engine_set_layout_mode;
    iface->get_layout_mode                 = (GWebEngineLayoutMode (*) (GWebEngine*))
        g_mozilla_engine_get_layout_mode;
    iface->set_load_images                 = (void (*) (GWebEngine*, GWebEngineImagePolicy))
        g_mozilla_engine_set_load_images;
    iface->get_load_images                 = (GWebEngineImagePolicy (*) (GWebEngine *self))
        g_mozilla_engine_get_load_images;
    iface->set_frame_stacking              = (void (*) (GWebEngine*, gboolean))
        g_mozilla_engine_set_frame_stacking;
    iface->get_frame_stacking              = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_get_frame_stacking;
    iface->save_web_page                   = (GWebStatus (*) (GWebEngine*, GWebEngineSaveAction, const gchar*))
        g_mozilla_engine_save_web_page;
    iface->save_current_frame              = (GWebStatus (*) (GWebEngine*, GWebEngineSaveAction, const gchar*))
        g_mozilla_engine_save_current_frame;
    iface->get_frame_title                 = (gchar* (*) (GWebEngine*, gint))
        g_mozilla_engine_get_frame_title;
    iface->get_element_details             = (gchar* (*) (GWebEngine*, gint))
        g_mozilla_engine_get_element_details;
    iface->get_frame_number                = (gint (*) (GWebEngine*))
        g_mozilla_engine_get_frame_number;
    iface->save_web_page_image             = (GWebStatus (*) (GWebEngine*, const gchar*, const gchar*))
        g_mozilla_engine_save_web_page_image;
    iface->select_all_text                 = (void (*) (GWebEngine*))
        g_mozilla_engine_select_all_text;
    iface->set_language_encoding           = (void (*) (GWebEngine*, GWebEngineEncoding))
        g_mozilla_engine_set_language_encoding;
    iface->set_frame_encoding              = (void (*) (GWebEngine*, GWebEngineEncoding))
        g_mozilla_engine_set_frame_encoding;
    iface->find_text                       = (gboolean (*) (GWebEngine*, const gchar*, gboolean, gboolean, gboolean, gboolean))
        g_mozilla_engine_find_text;
    iface->type_find_text                  = (GWebEngineSearchResult (*) (GWebEngine*, const gchar*, gboolean, gboolean, gboolean))
        g_mozilla_engine_type_find_text;
    iface->authenticate                    = (void (*) (GWebEngine*, gboolean, const gchar*, const gchar*))
        g_mozilla_engine_authenticate;
    iface->cut                             = (GWebStatus (*) (GWebEngine*))
        g_mozilla_engine_cut;
    iface->copy                            = (GWebStatus (*) (GWebEngine*))
        g_mozilla_engine_copy;
    iface->paste                           = (GWebStatus (*) (GWebEngine*))
        g_mozilla_engine_paste;
    iface->can_cut                         = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_can_cut;
    iface->can_paste                       = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_can_paste;
    iface->can_copy                        = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_can_copy;
    iface->can_selecttext                  = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_can_selecttext;
    iface->get_favicon_name                = (gchar* (*) (GWebEngine*, const gchar*))
        g_mozilla_engine_get_favicon_name;
    iface->get_engine_widget               = (GObject* (*) (GWebEngine*))
        g_mozilla_engine_get_engine_widget;
    iface->get_engine_notifier             = (GObject* (*) (GWebEngine*))
        g_mozilla_engine_get_engine_notifier;
    iface->set_open_link_mode              = (void (*) (GWebEngine*, gboolean))
        g_mozilla_engine_set_open_link_mode;
    iface->set_true_zoom_mode              = (GWebStatus (*) (GWebEngine*, gboolean))
        g_mozilla_engine_set_true_zoom_mode;
    iface->notify_plugins                  = (void (*) (GWebEngine*, gint))
        g_mozilla_engine_notify_plugins;
    iface->preload_current_skin            = (void (*) (GWebEngine*))
        g_mozilla_engine_preload_current_skin;
    iface->invoke_selectmatch              = (void (*) (GWebEngine*))
        g_mozilla_engine_invoke_selectmatch;
    iface->get_server_certificate          = (GArray* (*) (GWebEngine*, gpointer context))
        g_mozilla_engine_get_server_certificate;
    iface->get_page_verifier_name                 = (gchar* (*) (GWebEngine*))
        g_mozilla_engine_get_page_verifier_name;
     iface->permit_page_unload             = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_permit_page_unload;
    iface->get_thumbnail                   = (gboolean (*) (GWebEngine*, const gchar*, gint, gint))
        g_mozilla_engine_get_thumbnail;
    iface->get_text_zoom_level             = (gint (*) (GWebEngine*))
        g_mozilla_engine_get_text_zoom_level;
    iface->set_text_zoom_level             = (void (*) (GWebEngine*, gint))
        g_mozilla_engine_set_text_zoom_level;
    iface->enable_shistory_thumbnails      = (void (*) (GWebEngine*, gboolean, gint, gint))
        g_mozilla_engine_enable_shistory_thumbnails;
    iface->evaluate_js                     = (gboolean (*) (GWebEngine*, const gchar*))
        g_mozilla_engine_evaluate_js;
    iface->update_area                     = (void (*) (GWebEngine*, int, int, int, int, int, int))
        g_mozilla_engine_update_area;
    iface->send_mouse_event                = (void (*) (GWebEngine*, int, int, int, int, int, int, int))
        g_mozilla_engine_send_mouse_event;
    iface->realize_window                   = (gboolean (*) (GWebEngine*))
        g_mozilla_engine_realize_window;
}

static void
g_mozilla_engine_instance_init (GTypeInstance *instance, gpointer  g_class)
{
    TRACE_LOG();
    G_MOZILLA_ENGINE(instance)->engine = NULL;
    G_MOZILLA_ENGINE(instance)->shistory_listener = NULL;
    G_MOZILLA_ENGINE(instance)->find = NULL;
}

static void
g_mozilla_engine_finalize (GObject * instance)
{
    GMozillaEngine * self = G_MOZILLA_ENGINE (instance);
    g_return_if_fail (self);

    EAL_IF_GFREE_FUNC(self->suspend_timeouts_timer_id, g_source_remove);

    if (self->user_ctx) {
        EAL_IF_GFREE(self->user_ctx->ctx_url);
        EAL_IF_GFREE(self->user_ctx->ctx_objurl);
        EAL_IF_GFREE(self->user_ctx->ctx_docurl);
    }
    EAL_IF_GFREE(self->user_ctx);
    EAL_IF_GFREE(self->dom_ctx);
    EAL_IF_GFREE(self->mouse_ctx);

    GObjectClass *parent_class = 
            g_type_class_peek (g_type_parent (G_TYPE_MOZILLA_ENGINE));
    if (parent_class && parent_class->finalize)
        parent_class->finalize (instance);
}

GType
g_mozilla_engine_get_type (void)
{
    static GType type = 0;
    if (type == 0)
    {
        static const GTypeInfo info =
        {
            sizeof (GMozillaEngineClass),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
            (GClassInitFunc) g_mozilla_engine_class_init, /* class_init */
            NULL,   /* class_finalize */
            NULL,   /* class_data */
            sizeof (GMozillaEngine),
            0,      /* n_preallocs */
            g_mozilla_engine_instance_init    /* instance_init */
        };
        static const GInterfaceInfo iface_info =
        {
            (GInterfaceInitFunc) g_mozilla_engine_interface_init,
            NULL,   /* interface_finalize */
            NULL    /* interface_data */
        };
        type = g_type_register_static (G_TYPE_OBJECT,
                                       G_MOZILLA_ENGINE_NAME,
                                       &info, 0);
        g_type_add_interface_static (type,
                                     G_TYPE_WEB_ENGINE,
                                     &iface_info);
    }
    return type;
}
/* a macro for creating a new object of our type */
#define GET_MOZILLA_ENGINE_NEW (g_object_new(g_mozilla_engine_get_type(), NULL))

static void
g_mozilla_engine_class_init(GMozillaEngineClass *klass)
{
    TRACE_LOG();
    GObjectClass * gobject_class = (GObjectClass *) klass;
    
    // set up UI specific signals
    g_mozilla_engine_signals[URL_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_URL_CHANGED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, url_changed),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE, 1, G_TYPE_STRING);

    g_mozilla_engine_signals[TITLE_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_TITLE,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, title_changed),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE, 1, G_TYPE_STRING);

    g_mozilla_engine_signals[LOADING_PROGRESS]
        = g_signal_new (G_WEBWIDGET_SIGNAL_LOADING_PROGRESS,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, progress),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__INT_INT,
                        G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);

    g_mozilla_engine_signals[START_LOADING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_START_LOADING,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, start_loading),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE, 1, G_TYPE_STRING);

    g_mozilla_engine_signals[FINISHED_LOADING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FINISHED_LOADING,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, finished_loading),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__INT_INT_BOOL,
                        G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_INT, G_TYPE_BOOLEAN);

    g_mozilla_engine_signals[WINDOW_REQUEST]
        = g_signal_new (G_WEBWIDGET_SIGNAL_WINDOW_REQUEST,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, window_request),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__POINTER_STRING_UINT_BOOLEAN,
                        G_TYPE_NONE,
                        4,
                        G_TYPE_POINTER,
                        G_TYPE_STRING, G_TYPE_UINT, G_TYPE_BOOLEAN);

    g_mozilla_engine_signals[ENCODING]
        = g_signal_new (G_WEBWIDGET_SIGNAL_ENCODING,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, encoding),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1,
                        G_TYPE_UINT);

    g_mozilla_engine_signals[AUTHENTIFICATION_REQUIRED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_AUTHENTIFICATION_REQUIRED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass,
                                         authentification_required), NULL, NULL,
                        g_mozeal_marshal_VOID__INT_STRING_STRING,
                        G_TYPE_NONE, 3, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);

    g_mozilla_engine_signals[CONFIRM]
        = g_signal_new (G_WEBWIDGET_SIGNAL_CONFIRM,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, confirm),
                        NULL, NULL,
                        g_mozeal_marshal_BOOLEAN__STRING,
                        G_TYPE_BOOLEAN, 1, G_TYPE_STRING);

    g_mozilla_engine_signals[CONFIRM_EX]
        = g_signal_new (G_WEBWIDGET_SIGNAL_CONFIRM_EX,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, confirm_ex),
                        NULL, NULL,
                        g_mozeal_marshal_INT__STRING_STRING_UINT_STRING_STRING_STRING_STRING_POINTER,
                        G_TYPE_INT, 8,
                        G_TYPE_STRING,
                        G_TYPE_STRING,
                        G_TYPE_UINT,
                        G_TYPE_STRING,
                        G_TYPE_STRING,
                        G_TYPE_STRING,
                        G_TYPE_STRING,
                        G_TYPE_POINTER);

    g_mozilla_engine_signals[PROMPT]
        = g_signal_new (G_WEBWIDGET_SIGNAL_PROMPT,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, prompt),
                        NULL, NULL,
                        g_mozeal_marshal_STRING__STRING_STRING,
                        G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);

    g_mozilla_engine_signals[SELECT]
        = g_signal_new (G_WEBWIDGET_SIGNAL_SELECT,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, select),
                        NULL, NULL,
                        g_mozeal_marshal_BOOLEAN__STRING_STRING_POINTER_POINTER,
                        G_TYPE_BOOLEAN, 4, G_TYPE_STRING, G_TYPE_STRING,
                        G_TYPE_POINTER, G_TYPE_POINTER);

    g_mozilla_engine_signals[DOWNLOAD_REQUEST]
        = g_signal_new (G_WEBWIDGET_SIGNAL_DOWNLOAD_REQUEST,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, download_request),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__STRING_STRING_STRING_INT_INT,
                        G_TYPE_NONE, 5,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE, G_TYPE_INT,
                        G_TYPE_INT);

    g_mozilla_engine_signals[UPLOAD_DIALOG]
        = g_signal_new (G_WEBWIDGET_SIGNAL_UPLOAD_DIALOG,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, upload_dialog),
                        NULL, NULL,
                        g_mozeal_marshal_STRING__STRING_STRING,
                        G_TYPE_STRING, 2, G_TYPE_STRING, G_TYPE_STRING);

    g_mozilla_engine_signals[FILE_UPLOAD_PROGRESS]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FILE_UPLOAD_PROGRESS,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, file_upload_progress),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__INT_STRING,
                        G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_STRING);

    g_mozilla_engine_signals[WEBENGINE_POPUP_MENU]
        = g_signal_new (G_WEBWIDGET_SIGNAL_WEBENGINE_POPUP_MENU,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, webengine_popup_menu),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__UINT_INT_INT_STRING_STRING_STRING_STRING,
                        G_TYPE_NONE, 7, G_TYPE_UINT, G_TYPE_INT, G_TYPE_INT,
                        G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

    g_mozilla_engine_signals[MODAL_CERTIFICATE_DIALOG]
        = g_signal_new (G_WEBWIDGET_SIGNAL_MODAL_CERTIFICATE_DIALOG,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass,
                                         modal_certificate_dialog), NULL, NULL,
                        g_mozeal_marshal_INT__STRING_INT_INT_INT_INT_INT,
                        G_TYPE_INT, 6, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT,
                        G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);

    g_mozilla_engine_signals[ALERT]
        = g_signal_new (G_WEBWIDGET_SIGNAL_ALERT,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, alert),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1,
                        G_TYPE_STRING);

    g_mozilla_engine_signals[MAILTO]
        = g_signal_new (G_WEBWIDGET_SIGNAL_MAILTO,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, mailto),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1,
                        G_TYPE_STRING);

    g_mozilla_engine_signals[MODAL_DIALOG]
        = g_signal_new (G_WEBWIDGET_SIGNAL_MODAL_DIALOG,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, modal_dialog),
                        NULL, NULL,
                        g_mozeal_marshal_INT__STRING_STRING_INT_INT_INT_INT,
                        G_TYPE_INT,
                        6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
                        G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);

    g_mozilla_engine_signals[UNKNOWN_PROTOCOL]
        = g_signal_new (G_WEBWIDGET_SIGNAL_UNKNOWN_PROTOCOL,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, unknown_protocol),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__STRING_BOOL, G_TYPE_NONE, 2,
                        G_TYPE_STRING, G_TYPE_BOOLEAN);

    g_mozilla_engine_signals[RAISE_REQUEST]
        = g_signal_new (G_WEBWIDGET_SIGNAL_RAISE_REQUEST,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, raise_request),
                        NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

    g_mozilla_engine_signals[LOWER_REQUEST]
        = g_signal_new (G_WEBWIDGET_SIGNAL_LOWER_REQUEST,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, lower_request),
                        NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

    g_mozilla_engine_signals[FILE_UPLOAD_STARTED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FILE_UPLOAD_STARTED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, file_upload_started),
                        NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

    g_mozilla_engine_signals[FILE_UPLOAD_FINISHED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FILE_UPLOAD_FINISHED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, file_upload_finished),
                        NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

    g_mozilla_engine_signals[EXTERNAL_ACTION]
        = g_signal_new (G_WEBWIDGET_SIGNAL_EXTERNAL_ACTION,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, external_action),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1,
                        G_TYPE_STRING);

    g_mozilla_engine_signals[FRAME_STACKING_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_FRAME_STACKING_CHANGED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass,
                                         frame_stacking_changed), NULL, NULL,
                        g_cclosure_marshal_VOID__BOOLEAN, G_TYPE_NONE, 1,
                        G_TYPE_BOOLEAN);


    g_mozilla_engine_signals[SECURITY_MODE_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_SECURITY_MODE_CHANGED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, security_mode_changed),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1,
                        G_TYPE_UINT);

    g_mozilla_engine_signals[CLOSE_WINDOW]
        = g_signal_new (G_WEBWIDGET_SIGNAL_CLOSE_WINDOW,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, close_window),
                        NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN,
                        G_TYPE_NONE, 1, G_TYPE_BOOLEAN);

    g_mozilla_engine_signals[ICON_SIGNAL]
        = g_signal_new (G_WEBWIDGET_SIGNAL_ICON,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, icon),
                        NULL, NULL,
                        g_cclosure_marshal_VOID__STRING,
                        G_TYPE_NONE, 1, G_TYPE_STRING);

    g_mozilla_engine_signals[RSS_SIGNAL]
        = g_signal_new (G_WEBWIDGET_SIGNAL_RSS_REQUEST,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, rss_request),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__STRING_STRING,
                        G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);

    g_mozilla_engine_signals[ASK_COOKIE]
        = g_signal_new (G_WEBWIDGET_SIGNAL_ASK_COOKIE,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, ask_cookie),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__POINTER_INT_STRING_STRING_STRING_STRING_STRING_BOOLEAN_INT,
                        G_TYPE_NONE, 9,
                        G_TYPE_POINTER,
                        G_TYPE_INT,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
                        G_TYPE_BOOLEAN, G_TYPE_INT);

    g_mozilla_engine_signals[ON_SUBMIT_SIGNAL]
        = g_signal_new (G_WEBWIDGET_SIGNAL_ON_SUBMIT,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, on_submit),
                        NULL, NULL, g_mozeal_marshal_INT__VOID, G_TYPE_INT, 0);

    g_mozilla_engine_signals[SELECT_MATCH_SIGNAL]
        = g_signal_new (G_WEBWIDGET_SIGNAL_SELECT_MATCH,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, select_match),
                        NULL, NULL,
                        g_mozeal_marshal_INT__POINTER, G_TYPE_INT, 1, G_TYPE_POINTER);

    g_mozilla_engine_signals[KEY_EVENT_SIGNAL]
        = g_signal_new (G_WEBWIDGET_SIGNAL_KEY_EVENT,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, key_event),
                        NULL, NULL,
                        g_mozeal_marshal_BOOLEAN__UINT_UINT_UINT_UINT_POINTER, G_TYPE_BOOLEAN, 5, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_POINTER);

    g_mozilla_engine_signals[ON_SERVICE_NOTIFY]
        = g_signal_new (G_WEBWIDGET_SIGNAL_ON_SERVICE_NOTIFY,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, on_service_notify),
                        NULL, NULL,
                        g_mozeal_marshal_INT__POINTER_STRING_STRING,
                        G_TYPE_INT, 3, G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING);

    g_mozilla_engine_signals[PREF_WINDOW_SIZE]
        = g_signal_new (G_WEBWIDGET_SIGNAL_PREF_WINDOW_SIZE,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, size_request),
                        NULL, NULL,
                        g_mozeal_marshal_BOOLEAN__INT_INT_UINT_UINT,
                        G_TYPE_BOOLEAN,
                        4,
                        G_TYPE_INT, G_TYPE_INT,
                        G_TYPE_UINT, G_TYPE_UINT);

    g_mozilla_engine_signals[OPEN_URI]
        = g_signal_new (G_WEBWIDGET_SIGNAL_OPEN_URI,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, open_uri),
                        NULL, NULL,
                        g_mozeal_marshal_INT__STRING,
                        G_TYPE_INT, 1, G_TYPE_STRING);

    g_mozilla_engine_signals[MOZ_PAINT_NOTIFY]
        = g_signal_new (G_WEBWIDGET_SIGNAL_MOZ_PAINT_NOTIFY,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, moz_paint_notify),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__INT_INT_INT_INT_INT_INT_INT_INT,
                        G_TYPE_NONE, 8,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT);

    g_mozilla_engine_signals[SHM_BUFFER_INFO]
        = g_signal_new (G_WEBWIDGET_SIGNAL_SHM_BUFFER_INFO,
                        G_OBJECT_CLASS_TYPE (gobject_class),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, shm_buffer_info),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__ULONG_INT_INT_INT,
                        G_TYPE_NONE, 4,
                        G_TYPE_ULONG,
                        G_TYPE_INT,
                        G_TYPE_INT,
                        G_TYPE_INT);

    g_mozilla_engine_signals[NOTIFICATION_DIALOG]
        = g_signal_new (G_WEBWIDGET_SIGNAL_NOTIFICATION_DIALOG,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, notification_dialog),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__UINT_STRING,
                        G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_STRING);

    g_mozilla_engine_signals[HISTORY_CHANGED]
        = g_signal_new (G_WEBWIDGET_SIGNAL_HISTORY_CHANGED,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_FIRST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, history_changed),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__UINT_INT_INT,
                        G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_INT, G_TYPE_INT);

    g_mozilla_engine_signals[META_REFRESH]
        = g_signal_new (G_WEBWIDGET_SIGNAL_META_REFRESH,
                        G_TYPE_MOZILLA_ENGINE,
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET (GMozillaEngineClass, meta_refresh),
                        NULL, NULL,
                        g_mozeal_marshal_VOID__STRING_BOOLEAN_INT,
                        G_TYPE_NONE,
                        3, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT); 

    g_mozilla_engine_signals[ADD_SEARCH_PROVIDER]
         = g_signal_new (G_WEBWIDGET_ADD_SEARCH_PROVIDER,
                         G_TYPE_MOZILLA_ENGINE,
                         G_SIGNAL_RUN_LAST,
                         G_STRUCT_OFFSET (GMozillaEngineClass, add_search_provider),
                         NULL,
                         NULL,
                         g_mozeal_marshal_BOOLEAN__STRING,
                         G_TYPE_BOOLEAN,
                         1,
                         G_TYPE_STRING);

    G_OBJECT_CLASS (klass)->finalize = 
            (GObjectFinalizeFunc) g_mozilla_engine_finalize;
}

static void
g_mozilla_engine_destroy (GMozillaEngine *self)
{
    TRACE_LOG();
    g_mozilla_engine_reset (self);
}

GObject*
g_mozilla_engine_new (void)
{
    TRACE_LOG();
    return g_mozilla_engine_new_with_context (0);
}

GObject*
g_mozilla_engine_new_with_context (gpointer context)
{
    TRACE_LOG();
    return g_mozilla_engine_new_with_context_and_tracks (context, FALSE, FALSE, 0);
}

gboolean
content_set_cursor(GMozillaEngine *self, GdkCursorType type)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);

    if (!type)
        return FALSE;

    GdkCursor* cursor = NULL;

    GMozillaWeb *global = G_MOZILLA_WEB(self->global);

    switch(type)
    {
        case GDK_LEFT_PTR:
            if (global->default_cursor == NULL)
                global->default_cursor = gdk_cursor_new (GDK_LEFT_PTR);
            if (global->default_cursor)  
                cursor = global->default_cursor;   
            break;
        default:
            cursor = gdk_cursor_new (type);
    }

    if (cursor && self->engine)
    {
        GtkWidget * child = gtk_bin_get_child((GtkBin *)self->engine);
        if (child)
            gdk_window_set_cursor (GDK_WINDOW(GTK_WIDGET(child)->window), cursor);
        if (cursor != global->default_cursor)  
            gdk_cursor_unref(cursor);
        return TRUE;
    }

    return FALSE;
}

static void
context_changed_cb (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));

    if (self->dom_focus_while_loading) {
        // Don't reset context if we've received dom_focus while loading
        self->dom_focus_while_loading = FALSE;
        return;
    }

    g_mozilla_reset_context_info(self);

    self->ctx = G_OBJECT(gtk_microb_context_new());
    g_object_set_data(G_OBJECT(self->ctx), "mozembed", self->engine);
}

static void
location_changed_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));

    context_changed_cb(self);

    gchar * location = gtk_moz_embed_get_location(embed);

    if (!location)
        return;

    if (!strncmp(location, LOCALLY_CACHED_PAGE_URL_PREFIX, LOCALLY_CACHED_PAGE_URL_PREFIX_LEN)) {
        g_free(location);
        return;
    }

    g_object_set_data(G_OBJECT(self), "async", (void*)1);
    if (!strcmp(location, "") || !strcmp(location, "data:text/html,")) {
        g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[URL_CHANGED] , 0, "about:blank");
    } else {
        g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[URL_CHANGED] , 0, location);
    }

    g_free(location);
}

static void
title_changed_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    gchar * title = gtk_moz_embed_get_title(embed);
    if (!title)
        return;
    // bug 49447.- solved
    ULOG_DEBUG_F("Embed:%p, Title: '%s'", embed, title);
    g_object_set_data(G_OBJECT(self), "async", (void*)1);
    if (!strcmp(title, ""))
        g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[TITLE_CHANGED] , 0, "(Untitled)");
    else
        g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[TITLE_CHANGED] , 0, title);

    g_free(title);
}

static void
load_started_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    self->is_loading = TRUE;
    EAL_IF_GFREE(self->last_location);

    gchar * location = gtk_moz_embed_get_location(embed);

    self->mWindowSize.width = 0;
    self->mWindowSize.height = 0;

    self->last_net_status = 0;

    if (location) {
        EAL_IF_GFREE (self->will_open_location);
        self->will_open_location = g_strdup (location);
    }
    send_progress_signal(self, START_LOADING, self->will_open_location, 0, 0);
    EAL_IF_GFREE(location);
}

static void
load_finished_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    //ULOG_DEBUG_F("last_net_status: %i", self->last_net_status);
    self->is_loading = FALSE;

    GWebEngineEncoding encoding = g_mozilla_engine_get_encoding (embed);
    // (gint finish_status, gint page_size, gboolean has_frames)
    if (self->progress_bar_state != -1)
        location_changed_cb(embed, self);
    else
        context_changed_cb(self); // at least reset the context after loading
    send_progress_signal(self, FINISHED_LOADING, NULL, 0, 0);
    g_signal_emit_by_name (G_OBJECT(self), G_WEBWIDGET_SIGNAL_ENCODING, encoding);
    if (self->optimized_view)
    {
        g_mozilla_engine_observe("@mozilla.org/extensions/ftv;1", NULL, "optimized_view", NULL);
    }
    register_im_context(self);
    gint retval = 0;
    g_signal_emit (G_OBJECT (self), g_mozilla_engine_signals[ON_SERVICE_NOTIFY], 0, NULL, "grab_focus", NULL, &retval);
    gboolean result = NS_TO_GBOOLEAN(g_mozilla_engine_update_fixed_frames(self, TRUE));
    if (result != self->mFixedFramesAvailable) {
      ULOG("Fixed frames avail:%s", result ? "TRUE" : "FALSE");
      self->mFixedFramesAvailable = result;
      send_element_info(self, G_WEB_ELEMENT_INFO_ID_FIXED_FRAMES, 0, 0, 0, 0, 0, self->mFixedFramesAvailable);
    }

    if (retval)
        move_focus_to_engine(self);
}

static gboolean
meta_refresh_cb (GtkMozEmbed *embed, const gchar *refresh_url,
                 gboolean same_url,
                 gint milli_seconds,
                 GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), TRUE);

    // We are currently intrerested only in immediate refresh to another URI
    if ((0 != milli_seconds) || same_url)
      return TRUE;

    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[META_REFRESH], 0,
            refresh_url, same_url, milli_seconds );

    return TRUE;
}

static void
progress_change_cb (GtkMozEmbed *embed, gint cur,
                    gint max, GMozillaEngine *self)
{
    TRACE_LOG("cur: %i, max: %i", cur, max);
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    if (max > 0)
      self->page_size = max;
    else if (cur > 0)
      self->page_size = cur;
}

static void
progress_change_all_cb (GtkMozEmbed *embed, const char *uri,
                        gint cur, gint max, GMozillaEngine *self)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    TRACE_LOG("Uri: %s, cur: %i, max: %i, is_loading: %i, net_stat: %i", uri, cur, max, self->is_loading, self->last_net_status);
    if (max > cur && self->is_loading) {
        send_progress_signal(self, LOADING_PROGRESS, NULL, cur, max);
    } else {
        if (max >= cur) {
            send_progress_signal(self, FINISHED_LOADING, NULL, 0, 0);
            if (self->last_net_status != 0 && max > cur)
                self->last_net_status = 0;
        }
    }
}

static void
net_state_change_cb (GtkMozEmbed *embed, gint flags,
                     guint status, GMozillaEngine *self)
{
    TRACE_LOG();
}

static void
unknown_protocol_handle (GMozillaEngine *self)
{  
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    
    if (!self->user_ctx->ctx_url || !strcmp(self->user_ctx->ctx_url, ""))
        g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_UNKNOWN_PROTOCOL,
                self->will_open_location, FALSE);
    else
        g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_UNKNOWN_PROTOCOL,
              self->user_ctx->ctx_url, FALSE);
}

static void
net_state_change_all_cb (GtkMozEmbed *embed, const char *uri,
                         gint flags, guint status, GMozillaEngine *self)
{
    //TRACE_LOG("uri: '%s', flags: %x, status: %x", uri, flags, status);
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    // Network stuff
    switch (status)
    {
        case GTK_MOZ_EMBED_STATUS_FAILED_DNS:
            ULOG_DEBUG_F("GTK_MOZ_EMBED_STATUS_FAILED_DNS:");
        case GTK_MOZ_EMBED_STATUS_FAILED_CONNECT:
            ULOG_DEBUG_F("GTK_MOZ_EMBED_STATUS_FAILED_CONNECT:");
        case GTK_MOZ_EMBED_STATUS_PROXY_FAILED:
        {
            ULOG_DEBUG_F("GTK_MOZ_EMBED_STATUS_PROXY_FAILED:");
            break;
        }
    }
}

static void
link_message_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
}

static void
js_status_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
}

static void
new_window_cb (GtkMozEmbed *embed, GtkMozEmbed **newEmbed,
               guint chromemask, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    //CHROMEMASK IS MISSED// Fixme
    g_object_set_data(self->global, "chrome_flags", GUINT_TO_POINTER(chromemask));
    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[WINDOW_REQUEST] , 0, newEmbed, "", chromemask, FALSE);
    g_object_steal_data(self->global, "chrome_flags");
}

static void
visibility_cb (GtkMozEmbed *embed, gboolean visibility, GMozillaEngine *self)
{
    TRACE_LOG();
}

static void
destroy_brsr_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_object_set_data(G_OBJECT(self), "async", (void*)1);
    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[CLOSE_WINDOW] , FALSE, 0);
}

static gint
open_uri_cb (GtkMozEmbed *embed, const char *uri, GMozillaEngine *self)
{
    g_return_val_if_fail(uri && self, FALSE);

    gint blocked_uri = FALSE;
    gboolean after_click_async_emit = (self->emit_open_uri_after_click && self->user_iteraction_click);
    if (after_click_async_emit)
      g_object_set_data(G_OBJECT(self), "async", (void*)1);
    if (self->emit_open_uri_signal || after_click_async_emit)
      g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[OPEN_URI] , 0, uri, &blocked_uri);
    self->user_iteraction_click = FALSE;
    if (blocked_uri)
        return blocked_uri;

    EAL_IF_GFREE(self->will_open_location);
    self->will_open_location = g_strdup(uri);
    
    gboolean pr_local = protocol_is_local (self->will_open_location);

    // If the protocol is local, the url is going to be launched in other application
    // than browser. In that case we should notify the UI to stop showing connecting indicator
    // in toolbar; this is done by sending unknown protocol signal with setting only_notify -flag
    // as TRUE:
    if (pr_local)
    {
        g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_UNKNOWN_PROTOCOL,
                self->will_open_location, TRUE);
    }
    else
    {
        g_mozilla_web_force_online();
    }

    g_mozilla_engine_update_context_data(self, CTX_USER, embed, NULL);
    g_mozilla_engine_update_context_data(self, CTX_DOM, embed, NULL);
    g_mozilla_engine_update_context_data(self, CTX_MOUSE, embed, NULL);
    // don't interrupt anything
    return FALSE;
}

static void
update_im_context_mode(GMozillaEngine *self)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    TRACE_LOG("gtkIMContext:%p, multi:%i, pass:%i\n", self->gtkIMContext, ctx_multi(self), ctx_password(self));

    if (!self->gtkIMContext) return;
#ifdef USE_HILDON
    HildonGtkInputMode mode;
    g_object_get (G_OBJECT (self->gtkIMContext), "hildon-input-mode", &mode, NULL);
    if (ctx_multi(self))
        mode |= HILDON_GTK_INPUT_MODE_MULTILINE;
    else
        mode &= ~HILDON_GTK_INPUT_MODE_MULTILINE;
    if (ctx_password(self))
        mode |= HILDON_GTK_INPUT_MODE_INVISIBLE;
    else
        mode &= ~HILDON_GTK_INPUT_MODE_INVISIBLE;
    g_object_set (G_OBJECT (self->gtkIMContext), "hildon-input-mode", mode, NULL);
#endif
}

static gboolean
engine_focus_in_cb (GtkWidget *widget,
                    GdkEventFocus *event,
                    gpointer user_data)
{
    GMozillaEngine *self = (GMozillaEngine *) user_data;
    g_return_val_if_fail (G_MOZILLA_ENGINE(self), FALSE);

    // If the device has enabled virtual keyboard and text has been entered into it, the 
    // check_pending_im_enter_keyvent flag has been set up in g_moz_eal_im_context_commit.
    // If the user has closed the FKB with enter key, the IM Context will send a enter 
    // key event after calling gtk_im_context_focus_in.
    if (self->check_pending_im_enter_keyvent)
        gtk_im_context_focus_in (self->gtkIMContext);
    self->check_pending_im_enter_keyvent = FALSE;

    return FALSE;
}

static gboolean
gdk_key_press_cb (GtkWidget *embed, GdkEventKey *gdkevent,
                              GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(self && register_im_context(self) && gdkevent, FALSE);
    self->im_key_pressed = TRUE;

    if (gdkevent->keyval == 65312 && gdkevent->hardware_keycode == 135) // Workaround for Chr keys bug 78162
        self->im_do_manual_commit = TRUE;

    gint type;
    if (!g_mozilla_engine_get_pref (G_TYPE_INT, G_MOZILLA_PREF_ACCESSIBILITY_TABFOCUS, &type))
        type = G_MOZILLA_ACCESSIBILITY_TABFOCUS_DEFAULT;

    if (ctx_text_input_or_password(self))
    {
        if (gdkevent->keyval == GDK_Tab && type != 1) {
            type = 1;
            g_mozilla_engine_set_pref (G_TYPE_INT, G_MOZILLA_PREF_ACCESSIBILITY_TABFOCUS, &type);
        }
        gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);
        if (gtk_im_context_filter_keypress((GtkIMContext *)self->gtkIMContext, (GdkEventKey *)gdkevent))
        {
            return self->im_do_manual_commit;
        }

        if (gdkevent->keyval == GDK_Left || gdkevent->keyval == GDK_Right)
            gtk_im_context_reset ((GtkIMContext *)self->gtkIMContext);
    }

    if (type == 1) {
        type = G_MOZILLA_ACCESSIBILITY_TABFOCUS_DEFAULT;
        g_mozilla_engine_set_pref (G_TYPE_INT, G_MOZILLA_PREF_ACCESSIBILITY_TABFOCUS, &type);
    }

    return FALSE;
}

static gboolean
gdk_key_release_cb (GtkWidget *embed, GdkEventKey *gdkevent,
                              GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(self && (GtkIMContext *)self->gtkIMContext && gdkevent, FALSE);
    self->im_key_pressed = FALSE;

    if (gdkevent->keyval == 65312 && gdkevent->hardware_keycode == 135)
        self->im_do_manual_commit = FALSE;

    if (ctx_text_input_or_password(self))
    {
        gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);
        if (gtk_im_context_filter_keypress((GtkIMContext *)self->gtkIMContext, (GdkEventKey *)gdkevent))
        {
            return self->im_do_manual_commit;
        }
        if (gdkevent->keyval == GDK_Left || gdkevent->keyval == GDK_Right
           || gdkevent->keyval == GDK_Down || gdkevent->keyval == GDK_Up) {
            vkb_handle_action(self, self->user_ctx->common_data.moz_ctx_type
                || self->dom_ctx->common_data.moz_ctx_type, gdkevent);
            gtk_im_context_reset ((GtkIMContext *)self->gtkIMContext);
        }
    }

    return FALSE;
}

static gboolean
gdk_press_event_cb (GtkWidget *embed, GdkEventButton *event,
                              GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(self && (GtkIMContext *)self->gtkIMContext, FALSE);
    gtk_im_context_reset ((GtkIMContext *)self->gtkIMContext);
    self->mouse_down = TRUE;
    //self->moz_ctx_type = 0;
    gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);

    hildon_gtk_im_context_filter_event((GtkIMContext *)self->gtkIMContext, (GdkEvent *)event);

    return FALSE;
}

static gboolean
gdk_release_event_cb (GtkWidget *embed, GdkEventButton *event,
                              GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(self && event, FALSE);
    self->mouse_down = FALSE;

    ULOG_DEBUG_F("FKB Activate: is_input:%s\n", 
                 user_ctx_text_input_or_password(self)?"TRUE":"FALSE");

    if (user_ctx_text_input_or_password(self))
    {
        gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);

        update_im_context_mode(self);

        if (hildon_gtk_im_context_filter_event((GtkIMContext *)self->gtkIMContext, (GdkEvent *)event))
        {
            return TRUE;
        }
        return TRUE; // hildon_gtk_im_context_filter_event by some reasons always return FALSE and we cannot handle FKB properly...
    }

    return FALSE;
}

guint
ctx_mozembed_to_eal(guint moz_embed_ctx)
{
    TRACE_LOG();
    guint eal_ctx = GTK_MICROB_EAL_CONTEXT_NONE;
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_MULTILINE)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_MULTILINE:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_INPUT;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_FORM)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_FORM:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_FORM;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_NONE)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_NONE: ");
        eal_ctx = G_WEBENGINE_POPUPMENU_NOTYPE;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_XUL)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_XUL:");
        eal_ctx = G_WEBENGINE_POPUPMENU_NOTYPE;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_DOCUMENT)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_DOCUMENT:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_DOCUMENT;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_LINK)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_LINK:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_LINK;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_IMAGE)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_IMAGE:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_IMAGE;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_IFRAME)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_IFRAME:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_IFRAME;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_INPUT_TEXT:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_INPUT;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_IPASSWORD)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_IPASSWORD:");
        eal_ctx = G_WEBENGINE_POPUPMENU_PASSWORD;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_RICHEDIT)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_RICHEDIT:");
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_MAILTO)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_EMAIL:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_MAILTO;
        //	G_WEBENGINE_POPUPMENU_UNKNOWN_PROTOCOL;
    }
    if (moz_embed_ctx  & GTK_MICROB_EAL_CONTEXT_EMBED)
    {
        ULOG_INFO_F("CTXTYPE: GTK_MICROB_EAL_CONTEXT_EMBED:");
        eal_ctx |= G_WEBENGINE_POPUPMENU_EMBED;
    }
    return eal_ctx;
}


gboolean
g_moz_eal_im_context_has_selection(GtkIMContext *context, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    gboolean status = FALSE;
    gchar * selection = NULL;
    if ((self->user_ctx->common_data.ctx_type != G_WEBENGINE_POPUPMENU_NOTYPE)
        && (self->user_ctx->common_data.ctx_type != GTK_MICROB_EAL_CONTEXT_IPASSWORD)) {
        if (gtk_microb_context_get_text((GtkMicroBContext*)self->ctx, &selection, NULL) && selection)
            status = !!strcmp(selection, "");
    }
    EAL_IF_GFREE(selection);
    return status;
}

static void
g_moz_eal_im_context_clipboard_operation(GtkIMContext *context, int operation, GMozillaEngine *self)
{
    TRACE_LOG();

#ifdef USE_HILDON
    switch(operation)
    {
    case GTK_IM_CONTEXT_CLIPBOARD_OP_COPY:
        g_mozilla_engine_copy(self);
        break;
    case GTK_IM_CONTEXT_CLIPBOARD_OP_CUT:
        g_mozilla_engine_cut(self);
        break;
    case GTK_IM_CONTEXT_CLIPBOARD_OP_PASTE:
        g_mozilla_engine_paste(self);
        break;
    }
#endif
}

gboolean
g_moz_eal_im_context_commit(GtkIMContext *ctx, const gchar *str, GMozillaEngine *self)
{
    g_return_val_if_fail(str && G_MOZILLA_ENGINE(self), FALSE);
    TRACE_LOG();
    if (self->im_do_manual_commit || !self->im_key_pressed)
    gtk_microb_context_insert_text((GtkMicroBContext*)self->ctx, str,
                                gtk_microb_context_get_node((GtkMicroBContext*)self->ctx),
                                self->im_surr_enable?self->im_surr_offset:0,
                                self->im_surr_enable?self->im_surr_num:0);

    self->im_surr_enable = FALSE;
    if (!self->check_pending_im_enter_keyvent) {
      gtk_im_context_reset ((GtkIMContext *)self->gtkIMContext);
      gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);
    }
    self->is_vkb_showed = FALSE;
    return TRUE;
}

void
g_moz_eal_preedit_changed_cb(GtkIMContext *aContext, GMozillaEngine *self)
{
    TRACE_LOG();
    ULOG_DEBUG_F();
}

gboolean
g_moz_eal_im_context_delete_surrounding(GtkIMContext *aContext, gint arg1, gint arg2, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    self->im_surr_offset = arg1;
    self->im_surr_num = arg2;
    self->im_surr_enable = TRUE;
    return TRUE;
}

gboolean
g_moz_eal_im_context_retrieve_surrounding(GtkIMContext *aContext, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    
    gchar *string = NULL;
    gint cursor_pos = 0;

    gtk_microb_context_get_text((GtkMicroBContext*)self->ctx, (gchar**)&string, &cursor_pos);

    if (string != NULL)
    gtk_im_context_set_surrounding (aContext,
                                    string,
                                    strlen(string),
                                    cursor_pos);
    if (!self->is_vkb_showed)
      self->is_vkb_showed = TRUE;

    return TRUE;
}

static void
g_mozilla_unregister_im_context(GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    if (!self->gtkIMContext)
        return;
    if (self->im_signal_cb[IM_SIG_HAS_SELECTION])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_HAS_SELECTION]);
    if (self->im_signal_cb[IM_SIG_CLIPBOARD_OPERATION])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_CLIPBOARD_OPERATION]);
    if (self->im_signal_cb[IM_SIG_COMMIT])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_COMMIT]);

    if (self->im_signal_cb[IM_SIG_DELETE_SURROUNDING])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_DELETE_SURROUNDING]);
    if (self->im_signal_cb[IM_SIG_RETRIEVE_SURROUNDING])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_RETRIEVE_SURROUNDING]);

/*
    if (self->im_signal_cb[IM_SIG_PREEDIT_CHANGED])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_PREEDIT_CHANGED]);
    if (self->im_signal_cb[IM_SIG_PREEDIT_END])
       g_signal_handler_disconnect(self->gtkIMContext,
                                   self->im_signal_cb[IM_SIG_PREEDIT_END]);
    if (self->im_signal_cb[IM_SIG_PREEDIT_START])
        g_signal_handler_disconnect(self->gtkIMContext,
                                    self->im_signal_cb[IM_SIG_PREEDIT_START]);
*/
    gtk_im_context_set_client_window (self->gtkIMContext, NULL);
    gtk_im_context_reset(self->gtkIMContext);
    g_object_unref(self->gtkIMContext);
    self->gtkIMContext = NULL;
}

GtkIMContext*
register_im_context(GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && GTK_MOZ_EMBED(self->engine), NULL);
    if (self->gtkIMContext)
        return self->gtkIMContext;

    GtkIMContext * im_ctx = NULL;
    im_ctx = gtk_im_multicontext_new ();

    gtk_im_context_set_client_window (im_ctx, GDK_WINDOW(GTK_WIDGET(self->engine)->window));
    self->gtkIMContext = im_ctx;

#ifdef USE_HILDON
    HildonGtkInputMode mode;
    g_object_get (G_OBJECT(im_ctx), "hildon-input-mode", &mode, NULL);
    mode ^= HILDON_GTK_INPUT_MODE_AUTOCAP;
    g_object_set(G_OBJECT(im_ctx), "hildon-input-mode", mode, NULL);
#endif

    self->im_signal_cb[IM_SIG_HAS_SELECTION] =
        g_signal_connect(G_OBJECT(im_ctx), "has_selection", G_CALLBACK(g_moz_eal_im_context_has_selection), self);
    self->im_signal_cb[IM_SIG_CLIPBOARD_OPERATION] =
        g_signal_connect(G_OBJECT(im_ctx), "clipboard_operation", G_CALLBACK(g_moz_eal_im_context_clipboard_operation), self);
    self->im_signal_cb[IM_SIG_COMMIT] =
        g_signal_connect(G_OBJECT(im_ctx), "commit", G_CALLBACK(g_moz_eal_im_context_commit), self);

    self->im_signal_cb[IM_SIG_DELETE_SURROUNDING] =
        g_signal_connect(G_OBJECT(im_ctx), "delete-surrounding", G_CALLBACK(g_moz_eal_im_context_delete_surrounding), self);
    self->im_signal_cb[IM_SIG_RETRIEVE_SURROUNDING] =
        g_signal_connect(G_OBJECT(im_ctx), "retrieve-surrounding", G_CALLBACK(g_moz_eal_im_context_retrieve_surrounding), self);

/*
    self->im_signal_cb[IM_SIG_PREEDIT_CHANGED] =
        g_signal_connect(G_OBJECT(im_ctx), "preedit_changed", G_CALLBACK(g_moz_eal_im_context_preedit_changed_cb), self);
    self->im_signal_cb[IM_SIG_PREEDIT_END] =
        g_signal_connect(G_OBJECT(im_ctx), "preedit-end", G_CALLBACK(g_moz_eal_im_context_preedit_end), self);
    self->im_signal_cb[IM_SIG_PREEDIT_START] =
        g_signal_connect(G_OBJECT(im_ctx), "preedit-start", G_CALLBACK(g_moz_eal_im_context_preedit_start), self);
*/

    GtkWidget * child = gtk_bin_get_child(GTK_BIN(self->engine));
    if (child) {
        g_signal_connect(G_OBJECT(child), "focus-in-event",
                           G_CALLBACK(engine_focus_in_cb), self);
        g_signal_connect(G_OBJECT(child), "key_press_event",
                           G_CALLBACK(gdk_key_press_cb), self);
        g_signal_connect(G_OBJECT(child), "key_release_event",
                           G_CALLBACK(gdk_key_release_cb), self);
        g_signal_connect(G_OBJECT(child), "button_press_event",
                           G_CALLBACK(gdk_press_event_cb), self);
        g_signal_connect(G_OBJECT(child), "button_release_event",
                           G_CALLBACK(gdk_release_event_cb), self);
    }

    return self->gtkIMContext;
}

gboolean ctx_text_input_or_password(GMozillaEngine *self)
{
    return (ctx_text_input(self) || ctx_password(self));
}

gboolean user_ctx_text_input_or_password(GMozillaEngine *self)
{
    return (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT ||
    (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD));
}

gboolean ctx_text_input(GMozillaEngine *self)
{
    return (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT ||
            self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT);
}

gboolean ctx_password(GMozillaEngine *self)
{
    return (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD ||
            self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD);
}

gboolean ctx_multi(GMozillaEngine *self)
{
    return (self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_MULTILINE ||
            self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_MULTILINE);
}

gboolean dom_ctx_text_input_or_password(GMozillaEngine *self)
{
    return (self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT ||
            self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD);
}

static gboolean scroll_node(GMozillaEngine *self)
{
  g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
  gtk_microb_context_insert_text((GtkMicroBContext*)self->ctx, NULL,
                                 gtk_microb_context_get_node((GtkMicroBContext*)self->ctx),
                                 0, 0);
  EAL_IF_GFREE_FUNC(self->scroll_to_node_timer_id, g_source_remove);
  return FALSE;
}

static gboolean vkb_hide_timeout(GMozillaEngine *self)
{
  g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);

  if (!ctx_text_input(self)
     && !(self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_XUL)
     && !(self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_XUL))
     hildon_gtk_im_context_hide (register_im_context(self));
  EAL_IF_GFREE_FUNC(self->scroll_to_node_timer_id, g_source_remove);

  return FALSE;
}

static gboolean
vkb_handle_action(GMozillaEngine *self, guint moz_embed_ctx, GdkEventKey * event)
{
    TRACE_LOG();

    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    GtkIMContext *im = NULL;
    im = register_im_context(self);
    gboolean show_vkb = TRUE;
    
    if (event && (event->keyval != GDK_Return))
        show_vkb = FALSE;

    if (moz_embed_ctx & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT)
    {
        if (show_vkb) 
        {
            update_im_context_mode(self);

            self->vkb_shown = TRUE;
            hildon_gtk_im_context_show (im);
            EAL_IF_GFREE_FUNC(self->scroll_to_node_timer_id, g_source_remove);
            if (!self->xshm_rendering_enabled)
                self->scroll_to_node_timer_id = g_timeout_add(NODE_SCROLL_TIMEOUT, (GSourceFunc)scroll_node, self);
            return TRUE;
        }
    }
    self->scroll_to_node_timer_id = g_timeout_add(NODE_SCROLL_TIMEOUT, (GSourceFunc)vkb_hide_timeout, self);

    TRACE_LOG();
    return TRUE;
}

static gboolean
g_mozilla_engine_update_context_data (GMozillaEngine *self,
                                      guint ctx_type,
                                      GtkMozEmbed *embed,
                                      gpointer event)
{
    TRACE_LOG();

    GMozillaEngineCtxData* ctx_data = NULL;

    switch (ctx_type) {
        case CTX_USER: {
            ctx_data = &(self->user_ctx->common_data);
            break;
        }
        case CTX_DOM: {
            ctx_data = &(self->dom_ctx->common_data);
            break;
        }
        case CTX_MOUSE: {
            ctx_data = &(self->mouse_ctx->common_data);
            break;
        }
        default: {
            g_warning("UNKNOWN CTX TYPE!");
            return FALSE;
        }
    }

    if (event)
        self->last_event = event;

    gboolean multi = (ctx_data->moz_ctx_type & GTK_MICROB_EAL_CONTEXT_MULTILINE);
    gboolean input = (ctx_data->moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT);
    gboolean widget_hacked = !multi && input;
    ctx_data->moz_ctx_type = 0;
    if (embed && event) {
      gtk_microb_context_update_from_event((GtkMicroBContext*)self->ctx, event);
      if (ctx_type == CTX_USER)
        g_mozilla_engine_update_context_urls(self);
    }
    ctx_data->moz_ctx_type = GTK_MICROB_CONTEXT(self->ctx)->type;
    self->doc_index = GTK_MICROB_CONTEXT(self->ctx)->docindex;

    multi = (GTK_MICROB_CONTEXT(self->ctx)->type & GTK_MICROB_EAL_CONTEXT_MULTILINE);
    input = (GTK_MICROB_CONTEXT(self->ctx)->type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT);
    gboolean widget_need_hack = !multi && input;
    if (widget_hacked && !widget_need_hack)
        g_object_steal_data (self->engine, SINGLE_LINE_WIDGET_NAME);
    else if (!widget_hacked && widget_need_hack)
        g_object_set_data (self->engine, SINGLE_LINE_WIDGET_NAME, SINGLE_LINE_WIDGET_NAME);

    ctx_data->ctx_type = ctx_mozembed_to_eal(GTK_MICROB_CONTEXT(self->ctx)->type);
    update_im_context_mode(self);
    return TRUE;
}

static gboolean g_mozilla_engine_show_menu (GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);

    // if context sensitive context has no type, so there is no need to invoke it.
    if (self->user_ctx->common_data.ctx_type != G_WEBENGINE_POPUPMENU_NOTYPE)
    {
        gchar * selection = NULL;
        if (gtk_microb_context_get_text((GtkMicroBContext*)self->ctx, &selection, NULL) && selection)
        {
            if (0 == strcmp(selection, ""))
            {
                selection = NULL;
                if (ctx_text_input(self))
                    return FALSE;
            }
        }

        if (selection == NULL && 
            self->user_ctx->common_data.ctx_type & G_WEBENGINE_POPUPMENU_LINK) {

            // If we're dealing with a link and there's no selection, we'll pass
            // the text contents of the anchor tag to UI.
            // This is used by add bookmark context menu item as the default
            // bookmark name.
            selection = g_mozilla_get_node_text (
                    gtk_microb_context_get_node ((GtkMicroBContext*)self->ctx));
        }

        // NOTE: Hack for UI
        const gchar *frame_url = NULL;
        if (self->doc_index != -1 || self->user_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IFRAME)
            frame_url = self->user_ctx->ctx_docurl;
        else
            frame_url = ""; //UI hack
        gdk_pointer_ungrab(GDK_CURRENT_TIME);
        g_signal_emit(G_OBJECT(self),
                      g_mozilla_engine_signals[WEBENGINE_POPUP_MENU], 0,
                      self->user_ctx->common_data.ctx_type,
                      self->user_ctx->x,
                      self->user_ctx->y,
                      selection,
                      self->user_ctx->ctx_url,
                      self->user_ctx->ctx_objurl,
                      frame_url);
        EAL_IF_GFREE(selection);
        self->menu_shown = TRUE;
        return TRUE;
    }
    return FALSE;
}

static gint
dom_key_down_cb (GtkMozEmbed *embed, void *event,
                 GMozillaEngine *self)
{
    TRACE_LOG();
    gint key_handle = g_mozilla_handle_key_event (event);
    gboolean val = FALSE;
    if (key_handle && self->prev_eventkey)
    {
        GdkEventKey *gdk_event_key = self->prev_eventkey;
        g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[KEY_EVENT_SIGNAL] , 0, gdk_event_key->type, gdk_event_key->keyval, gdk_event_key->state, gdk_event_key->hardware_keycode, gdk_event_key->window, &val);
    }
    register_im_context(self);
    user_iteraction_happens(self);
    if (self->prev_eventkey)
    {
        gdk_event_free((GdkEvent*)self->prev_eventkey);
        self->prev_eventkey = NULL;
    }
    return val;
}

static gint
dom_key_press_cb (GtkMozEmbed *embed, void *event,
                  GMozillaEngine *self)
{
    TRACE_LOG();
    register_im_context(self);
    return FALSE;
}

static gint
dom_key_up_cb (GtkMozEmbed *embed, void *event,
               GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && event, FALSE);
    if (self->fake_key_id) return FALSE;
    g_mozilla_simple_navigation(self, event);
    g_mozilla_engine_update_context_data(self, CTX_DOM, embed, event);
    vkb_handle_action(self, self->dom_ctx->common_data.moz_ctx_type, event);
    return FALSE;
}

static gint
dom_mouse_down_cb (GtkMozEmbed *embed, void *event,
                   GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && event, FALSE);
    user_iteraction_happens(self);
    self->dom_ctx->mouse_scroll = FALSE;
    self->menu_shown = FALSE;
    register_im_context(self);
    g_mozilla_engine_update_context_data(self, CTX_DOM, embed, event);

    self->dom_ctx->mouse_up_received = FALSE;

   if (self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT ||
       self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD) {
       gtk_im_context_reset ((GtkIMContext *)self->gtkIMContext);
       gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);
       GdkEvent *gdk_event = gdk_event_new (GDK_BUTTON_PRESS);

       hildon_gtk_im_context_filter_event((GtkIMContext *)self->gtkIMContext, (GdkEvent *)gdk_event);

       gdk_event_free (gdk_event);
   }

    // absorb quickly repeated taps
    gettimeofday (&current_time_press, NULL);
    g_mozilla_support_timeval_subtract (&result, &current_time_press, &former_time_press);
    former_time_press = current_time_press;
    if (result.tv_usec <= 350000 && !result.tv_sec)
    {
        return FALSE;
    }

    return TRUE;
}

static gboolean vkb_up_timeout_cb(GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    
    if (self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_INPUT_TEXT ||
        self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_IPASSWORD) {
        gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);
        update_im_context_mode(self);
        GdkEvent *gdk_event = gdk_event_new (GDK_BUTTON_RELEASE);
#ifdef USE_HILDON
        hildon_gtk_im_context_filter_event((GtkIMContext *)self->gtkIMContext, (GdkEvent *)gdk_event);
#endif
        gdk_event_free (gdk_event);
    }
    
    EAL_IF_GFREE_FUNC (self->vkb_up_timeout, g_source_remove);
    self->dom_ctx->mouse_scroll = FALSE;
    return FALSE;
}

static gint dom_mouse_up_cb (GtkMozEmbed *embed, void *event,
                             GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && embed && event, FALSE);
    self->dom_ctx->mouse_up_received = TRUE;
    self->menu_shown = FALSE;
    // Fixed most of problems with iteractions after IM visiting
    if (!self->is_vkb_showed && !self->dom_ctx->mouse_scroll)
        g_mozilla_engine_update_context_data(self, CTX_DOM, embed, event);
    if (!self->vkb_up_timeout)
        self->vkb_up_timeout = g_timeout_add (NODE_SCROLL_TIMEOUT, (GSourceFunc) vkb_up_timeout_cb, self);
    return FALSE;
}

#ifdef UNUSED
static gint dom_mouse_move_cb (GtkMozEmbed *embed, void *event,
                               GMozillaEngine *self)
{
    TRACE_LOG();
    return FALSE;
}
#endif

static gint dom_mouse_scroll_cb (GtkMozEmbed *embed, void *event,
                                 GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    self->dom_ctx->mouse_scroll = TRUE;
    if (self->dom_ctx->common_data.moz_ctx_type & GTK_MICROB_EAL_CONTEXT_DOCUMENT)
    {
        return FALSE;
    }

    TRACE_LOG("Scrolling canceled");
    return TRUE;
}

static gboolean
show_menu_cb (gpointer data)
{
    TRACE_LOG();
    GMozillaEngine *self = G_MOZILLA_ENGINE(data);
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    if (!self->dialog_is_active && self->mouse_down && !self->menu_shown)
        g_mozilla_engine_show_menu(self);
    return FALSE;
}

gint dom_mouse_long_press_cb (GtkMozEmbed *embed, void *event,
                                     GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && embed && event, FALSE);
    user_iteraction_happens(self);
    g_mozilla_engine_update_context_data(self, CTX_DOM, embed, event);
    g_idle_add(show_menu_cb, self);
    return FALSE;
}

gint dom_context_menu_cb (GtkMozEmbed *embed, void *event,
                          GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && embed && event, FALSE);
    user_iteraction_happens(self);
    g_mozilla_engine_update_context_data(self, CTX_USER, embed, event);
    self->menu_shown = FALSE;
    g_idle_add(show_menu_cb, self);
    return FALSE;
}


static gint dom_focus_cb (GtkMozEmbed *embed, void *event,
                          GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail (G_IS_MOZILLA_ENGINE(self), FALSE);
    GWebEnginePopupMenuType type = G_WEBENGINE_POPUPMENU_NOTYPE;
    send_focused_element_message (self, &type);
    if (type & G_WEBENGINE_POPUPMENU_INPUT) {
      EAL_IF_GFREE_FUNC(self->fake_key_id, g_source_remove);
      self->fake_key_id = g_idle_add((GSourceFunc)send_fake_key_event_to_layout, self);

      if (!self->dom_ctx->common_data.ctx_type && !self->dom_ctx->common_data.moz_ctx_type) {
          register_im_context(self);
          gtk_im_context_focus_in((GtkIMContext *)self->gtkIMContext);
      }

      g_mozilla_engine_update_context_data(self, CTX_DOM, embed, event);

      if (!user_ctx_text_input_or_password(self))
          self->dom_focused_input = dom_ctx_text_input_or_password(self);
    }

    self->dom_focus_while_loading = self->is_loading;
    return FALSE;
}


static gint dom_mouse_click_cb (GtkMozEmbed *embed, void *event,
                                GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    if (!self->dom_ctx->mouse_up_received)
        return dom_mouse_up_cb(embed, event, self);
    return FALSE;
}

static gint dom_mouse_dbl_click_cb (GtkMozEmbed *embed, void *event,
                                    GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    return FALSE;
}

static gint dom_mouse_over_cb (GtkMozEmbed *embed, void *event,
                               GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    if (!self->is_vkb_showed)
        g_mozilla_engine_update_context_data(self, CTX_MOUSE, embed, event);
    get_mouse_over_element_info(self, gtk_microb_context_get_node((GtkMicroBContext*)self->ctx));
    return FALSE;
}

static gint dom_mouse_out_cb (GtkMozEmbed *embed, void *event,
                              GMozillaEngine *self)
{
    //    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    self->mouse_down = FALSE;
    return FALSE;
}

static gint dom_activate_cb (GtkMozEmbed *embed, void *event,
                             GMozillaEngine *self)
{
    TRACE_LOG();
    return FALSE;
}

static gint dom_focus_in_cb (GtkMozEmbed *embed, void *event,
                             GMozillaEngine *self)
{
    TRACE_LOG();
    return FALSE;
}

static gint dom_focus_out_cb (GtkMozEmbed *embed, void *event,
                              GMozillaEngine *self)
{
    TRACE_LOG();
    return FALSE;
}

static gint dom_window_blur_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[LOWER_REQUEST] , 0);
    return TRUE;
}

static gint dom_window_focus_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[RAISE_REQUEST] , 0); //UI buggy and not ready for non direct widget
    return TRUE;
}

static void dom_content_blocked_cb (GtkMozEmbed *embed, const char *uri, const char *host, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    GtkMozEmbed *newEmbed = NULL;
    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[WINDOW_REQUEST] , 0, &newEmbed, uri, 0, TRUE);
    if (newEmbed)
        g_warning ("Popup functionality's broken, created unnecessary object!, uri:'%s', host:'%s'\n", uri, host);
}

static void
size_to_cb (GtkMozEmbed *embed, gint width, gint height,
            GMozillaEngine *self)
{
    TRACE_LOG("*** size_to_cb %d %d", width, height);
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    gboolean retval = FALSE;
    g_signal_emit(G_OBJECT(self), g_mozilla_engine_signals[PREF_WINDOW_SIZE] , 0, width, height, 0, 0, &retval);
    if (!retval)
        g_mozilla_reflow_content(embed);
}

static gboolean
upload_dialog_cb (GtkMozEmbed *embed, const gchar *path, const gchar *filter, gchar **file_name_with_path, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && filter && file_name_with_path, FALSE);
    gchar *response = NULL;

    g_signal_emit_by_name (G_OBJECT(self), G_WEBWIDGET_SIGNAL_UPLOAD_DIALOG, getenv("HOME"), filter, &response);
    *file_name_with_path = response;

    return response!=NULL;
}

static void
file_upload_started_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_signal_emit_by_name (G_OBJECT(self), G_WEBWIDGET_SIGNAL_FILE_UPLOAD_STARTED);
}

static void
file_upload_finished_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_signal_emit_by_name (G_OBJECT(self), G_WEBWIDGET_SIGNAL_FILE_UPLOAD_FINISHED);
}

/* Implementation moved to gmozillaweb
prompt_cb confirm_ex_cb prompt_auth_cb select_cb
*/
gint
g_mozilla_engine_alert (GMozillaEngine * self, const gchar * dialog_title,
          const gchar * alert_msg, gboolean is_blocking,
          const char *check_msg, gboolean * check_val)
{

    TRACE_LOG ();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);

    ULOG_DEBUG_F ("dialog_title: '%s', alert_msg: '%s'", dialog_title,
                  alert_msg);
    if (alert_msg && dialog_title
        && (g_ascii_strcasecmp (dialog_title, "Unknown Protocol") == 0))
    {
        unknown_protocol_handle (self);
        load_finished_cb (GTK_MOZ_EMBED (self->engine), self);
        return TRUE;
    }
    
    // set flag to prevent any forbidden actions during popup
    self->dialog_is_active = TRUE;
    
    if (alert_msg && dialog_title
        && g_strstr_len (dialog_title, strlen (dialog_title), "JavaScript"))
    {
        gchar *message = g_strdup_printf ("jalert:%s", alert_msg);
        g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_ALERT,
                               message);
        g_free (message);
    }
    else if (alert_msg)
        g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_ALERT,
                               alert_msg);

    // popup has been dismissed by now
    self->dialog_is_active = FALSE;

    if (!self->is_loading)
    {
        g_object_set_data(G_OBJECT(self), "async", (void*)1);
        g_signal_emit (G_OBJECT (self), g_mozilla_engine_signals[FINISHED_LOADING], 0,
                               self->last_net_status, self->page_size, FALSE);
        content_set_cursor (self, GDK_LEFT_PTR);
    }
    // Hack to disable mouse_down fake event
    return FALSE;
}

gboolean
g_mozilla_engine_confirm (GMozillaEngine * self, const gchar * dialog_title,
            const gchar * dialog_msg, const char *check_msg,
            gboolean * check_val)
{

    TRACE_LOG ();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && dialog_msg, FALSE);

    // set flag to prevent any forbidden actions during popup
    self->dialog_is_active = TRUE;
    
    gboolean ret_val;
    g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_CONFIRM,
                           dialog_msg, &ret_val, NULL);

    // popup has been dismissed by now
    self->dialog_is_active = FALSE;

    if (!self->is_loading)
    {
        g_object_set_data(G_OBJECT(self), "async", (void*)1);
        g_signal_emit (G_OBJECT (self), g_mozilla_engine_signals[FINISHED_LOADING], 0,
                               self->last_net_status, self->page_size, FALSE);
        content_set_cursor (self, GDK_LEFT_PTR);
    }

    return ret_val;
}

static void
destroy_cb (GtkMozEmbed *embed, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
//    if (!G_MOZILLA_WEB(self->global)->no_xembed)
      self->engine = NULL;
    g_mozilla_engine_reset (self);
}

static void
rss_request_cb (GtkMozEmbed *embed, gchar *url, gchar *title, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_object_set_data(G_OBJECT(self), "async", (void*)1);
    g_signal_emit_by_name (G_OBJECT (self), G_WEBWIDGET_SIGNAL_RSS_REQUEST, url, title);
    if (self->progress_bar_state == -1)
        send_progress_signal(self, FINISHED_LOADING, NULL, 0, 0);
}

static void
icon_changed_cb (GtkMozEmbed *embed, gchar* aPixBuf, GMozillaEngine *self)
{
    TRACE_LOG("now send icon signal to UI %s\n", aPixBuf);
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    // foward signal to UI ??
#ifdef MOZEAL_LOGGING
    ULOG_DEBUG_F("icon_changed_cb is called in new EAL \n");
    if (aPixBuf == NULL)
        ULOG_DEBUG_F("PixBuf is NULL \n");
    else
        ULOG_DEBUG_F("PixBuf is NOT NULL \n");
    ULOG_DEBUG_F("now send icon signal to UI \n");
#endif
#ifndef USE_OS2006
    g_signal_emit_by_name (G_OBJECT (self), "icon", aPixBuf);
#endif
}

static void
security_mode_changed_cb (GtkMozEmbed *embed, gpointer request, guint state, GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    unsigned int sec_mode = self->sec_mode;
    self->sec_mode = g_mozilla_engine_get_security_mode(state);
    if (sec_mode != self->sec_mode)
      g_signal_emit_by_name(G_OBJECT(self), G_WEBWIDGET_SIGNAL_SECURITY_MODE_CHANGED, self->sec_mode);
}

static void
network_error_cb(GtkMozEmbed *embed, const gint code, const gchar *status, const gchar *url, GMozillaEngine *self)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    ULOG_DEBUG_F("embed: %p, code: %i, status: '%s', url: '%s'", embed, code, status, url);    
    self->last_net_status = G_WEBENGINE_LOADING_FINISH_UNKNOWN;
}

static gint
on_service_notify_cb(GtkMozEmbed *embed, gpointer ctx, const gchar *topic, const gchar *data, GMozillaEngine *self)
{
    TRACE_LOG();
    gint retval = 0;
    g_return_val_if_fail(G_MOZILLA_ENGINE(self), retval);
    g_signal_emit (G_OBJECT (self), g_mozilla_engine_signals[ON_SERVICE_NOTIFY], 0,
                   ctx, topic, data, &retval);
    return retval;
}

static void
g_mozilla_engine_array_insert_signal(GMozillaEngine *self, guint signal)
{
    g_return_if_fail(G_MOZILLA_ENGINE(self));
    g_array_insert_val(self->signals_array, self->signals_array_len, signal);
    self->signals_array_len++;
}

static gboolean
g_mozilla_engine_signals_connect(GMozillaEngine *self)
{
    TRACE_LOG();
    g_return_val_if_fail(G_MOZILLA_ENGINE(self) && self->engine, FALSE);
    GObject * engine = G_OBJECT(self->engine);
    self->signals_array =
        g_array_new (FALSE, FALSE, sizeof (gint));

    guint val;
    val = g_signal_connect(G_OBJECT(engine), "location",
                           G_CALLBACK(location_changed_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hook up the title change to update the window title
    val = g_signal_connect(G_OBJECT(engine), "title",
                           G_CALLBACK(title_changed_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hook up the start and stop signals
    val = g_signal_connect(G_OBJECT(engine), "net_start",
                           G_CALLBACK(load_started_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "net_stop",
                           G_CALLBACK(load_finished_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to changes in progress
    val = g_signal_connect(G_OBJECT(engine), "progress",
                           G_CALLBACK(progress_change_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "progress_all",
                           G_CALLBACK(progress_change_all_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hook up to the change in network status
    val = g_signal_connect(G_OBJECT(engine), "net_state",
                           G_CALLBACK(net_state_change_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "net_state_all",
                           G_CALLBACK(net_state_change_all_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to changes in over-link message
    val = g_signal_connect(G_OBJECT(engine), "link_message",
                           G_CALLBACK(link_message_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to changes in js status message
    val = g_signal_connect(G_OBJECT(engine), "js_status",
                           G_CALLBACK(js_status_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to see whenever a new window is requested
    val = g_signal_connect(G_OBJECT(engine), "new_window",
                           G_CALLBACK(new_window_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to any requested visibility changes
    val = g_signal_connect(G_OBJECT(engine), "visibility",
                           G_CALLBACK(visibility_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to the signal that says that the browser requested to be
    // destroyed
    val = g_signal_connect(G_OBJECT(engine), "destroy_browser",
                           G_CALLBACK(destroy_brsr_cb), self);

    // hookup to the signal that is called when someone clicks on a link
    // to load a new uri
    val = g_signal_connect(G_OBJECT(engine), "open_uri",
                           G_CALLBACK(open_uri_cb), self);
    // this signal is emitted when there's a request to change the
    // containing browser window to a certain height, like with width
    // and height args for a window.open in javascript
    val = g_signal_connect(G_OBJECT(engine), "size_to",
                           G_CALLBACK(size_to_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // Emitted when a page performs immediate redirection with meta refresh
    val = g_signal_connect(G_OBJECT(engine), "meta_refresh",
                           G_CALLBACK(meta_refresh_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // key event signals
    val = g_signal_connect(G_OBJECT(engine), "dom_key_down",
                           G_CALLBACK(dom_key_down_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_key_press",
                           G_CALLBACK(dom_key_press_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_key_up",
                           G_CALLBACK(dom_key_up_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_down",
                           G_CALLBACK(dom_mouse_down_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_up",
                           G_CALLBACK(dom_mouse_up_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);
#ifdef UNUSED
    g_signal_connect(G_OBJECT(engine), "dom_mouse_move",
                G_CALLBACK(dom_mouse_move_cb), self);
#endif
    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_scroll",
                           G_CALLBACK(dom_mouse_scroll_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_long_press",
                           G_CALLBACK(dom_mouse_long_press_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_context_menu",
                           G_CALLBACK(dom_context_menu_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);
    
    val = g_signal_connect(G_OBJECT(engine), "dom_focus",
                           G_CALLBACK(dom_focus_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_click",
                           G_CALLBACK(dom_mouse_click_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_dbl_click",
                           G_CALLBACK(dom_mouse_dbl_click_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_over",
                           G_CALLBACK(dom_mouse_over_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_mouse_out",
                           G_CALLBACK(dom_mouse_out_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_activate",
                           G_CALLBACK(dom_activate_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_focus_in",
                           G_CALLBACK(dom_focus_in_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_focus_out",
                           G_CALLBACK(dom_focus_out_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "upload_dialog",
                           G_CALLBACK(upload_dialog_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "file_upload_started",
                           G_CALLBACK(file_upload_started_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "file_upload_finished",
                           G_CALLBACK(file_upload_finished_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    // hookup to when the window is destroyed
    val = g_signal_connect(G_OBJECT(engine), "destroy",
                           G_CALLBACK(destroy_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "security_change",
                           G_CALLBACK(security_mode_changed_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "icon_changed",
                           G_CALLBACK(icon_changed_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "network_error",
                           G_CALLBACK(network_error_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "rss_request",
                           G_CALLBACK(rss_request_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_window_focus",
                           G_CALLBACK(dom_window_focus_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_window_blur",
                           G_CALLBACK(dom_window_blur_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "dom_content_blocked",
                           G_CALLBACK(dom_content_blocked_cb), self);
    g_mozilla_engine_array_insert_signal(self, val);

    val = g_signal_connect(G_OBJECT(engine), "on_service_notify",
                           G_CALLBACK(on_service_notify_cb), self);

    return TRUE;
}

static gboolean
g_mozilla_engine_signals_disconnect(GMozillaEngine *self)
{
    TRACE_LOG();

    g_return_val_if_fail(G_MOZILLA_ENGINE(self), FALSE);
    
    if (!self->engine)
        return FALSE;

    gint i = 0;
    guint signal;
    for (i = 0; i < self->signals_array_len; i++)
    {
        signal = g_array_index(self->signals_array, gint, i);
        g_signal_handler_disconnect(G_OBJECT(self->engine), signal);
    }
    if (self->signals_array)
    {
        g_array_free(self->signals_array, TRUE);
        self->signals_array = NULL;
    }
    self->signals_array_len = 0;
    return TRUE;
}

GObject*
g_mozilla_engine_new_with_context_and_tracks (gpointer context,
                                              gboolean leave_tracks,
                                              gboolean without_history,
                                              guint32 flags)
{

    TRACE_LOG();
    GObject* instance = G_OBJECT(GET_MOZILLA_ENGINE_NEW);

    GMozillaEngine *self = G_MOZILLA_ENGINE(instance);
    g_return_val_if_fail(self, NULL);

    self->user_ctx = g_new0(GMozillaEngineCtxUserData, 1);
    self->dom_ctx = g_new0(GMozillaEngineCtxDomData, 1);
    self->mouse_ctx = g_new0(GMozillaEngineCtxMouseData, 1);

    const char *shm_env = getenv("MOZ_ENABLE_XSHM");
    self->xshm_rendering_enabled = (shm_env && g_str_has_prefix(shm_env, "1"));
    const char *no_xembed_env = getenv(G_MOZILLA_MOZ_DISABLE_XEMBED);
    gboolean not_realize = no_xembed_env && g_str_has_prefix(no_xembed_env, "1");
    g_mozilla_engine_set_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_WIDGET_NOT_REALIZED, &not_realize);

    if (context)
    {
        GtkMozEmbed **newEmbed = (GtkMozEmbed **)context;
        if (*newEmbed) {
            self->engine = G_OBJECT(*newEmbed);
        } else {
            self->engine = G_OBJECT(gtk_moz_embed_new()); //CHROMEMASK IS MISSED
            *newEmbed = GTK_MOZ_EMBED(self->engine);
        }
    }
    else if (leave_tracks && !without_history)
    {
        self->engine = G_OBJECT(gtk_moz_embed_new());
    }
    else if (leave_tracks && without_history)
    {
        self->engine = G_OBJECT(gtk_moz_embed_new());
    }
    else
    {
        self->engine = G_OBJECT(gtk_moz_embed_new());
    }
    if (flags)
        gtk_moz_embed_set_chrome_mask((GtkMozEmbed*)self->engine, flags);

    g_mozilla_web_force_online();
    self->ctx = G_OBJECT(gtk_microb_context_new());
    g_object_set_data(G_OBJECT(self->ctx), "mozembed", self->engine);
    g_object_set_data(G_OBJECT(self->engine), G_MOZILLA_EAL_CTX_TYPE, &self->user_ctx->common_data.moz_ctx_type);
    g_object_set_data(G_OBJECT(self->engine), G_MOZILLA_EAL_PARENT, self);
    GtkWidget * mozCont = g_mozilla_engine_get_mozcontainer(self);
    if (mozCont)
        g_object_set_data(G_OBJECT(mozCont), G_MOZILLA_EAL_PARENT, self);

    self->emit_open_uri_signal = (getenv ("MOZEAL_OPENURI")) ? TRUE : FALSE;
    self->emit_open_uri_after_click = (getenv ("MOZEAL_OPENURI_ONCLICK")) ? TRUE : FALSE;
    self->user_iteraction_click = FALSE;
    self->full_update_required = TRUE;

    self->signals_array = NULL;
    self->signals_array_len = 0;
    g_mozilla_engine_signals_connect(self);
    self->doc_index = -1;

    self->is_vkb_showed = FALSE;
    self->last_event = NULL;
    self->last_net_status = 0;
    self->sec_mode = G_WEBENGINE_NO_SECURITY;
    self->gtkIMContext = NULL;
    self->current_zoom = 100;
    self->optimized_view = FALSE;
    self->will_open_location = NULL;
    self->waiting_connection = FALSE;
    self->im_surr_offset = 0;
    self->im_surr_num = 0;
    self->im_surr_enable = FALSE;
    self->user_iteraction_happens = TRUE;
    self->progress_bar_state = -1;
    self->extended_im_mode = GDK_EXTENSION_EVENTS_NONE;
    self->page_size = 0;
    self->scroll_to_node_timer_id = 0;
    self->vkb_up_timeout = 0;
    self->mouse_down = TRUE;
    self->vkb_shown = FALSE;
    self->key_press_time.tv_sec = 0;
    self->key_press_time.tv_usec = 0;
    self->key_release_time.tv_sec = 0;
    self->key_release_time.tv_usec = 0;
    self->im_do_manual_commit = FALSE;
    self->im_key_pressed = FALSE;
    self->mZoomValue = 1;
    self->mLastVisibleArea.x = 0;
    self->mLastVisibleArea.y = 0;
    self->mLastVisibleArea.width = 0;
    self->mLastVisibleArea.height = 0;

    self->mWindowSize.x = 0;
    self->mWindowSize.y = 0;
    self->mWindowSize.width = 800;
    self->mWindowSize.height = 480;

    self->mPrevScrollPos.x = 0;
    self->mPrevScrollPos.y = 0;

    self->mMinWindowSize.x = 0;
    self->mMinWindowSize.y = 0;
    self->mMinWindowSize.width = 0;
    self->mMinWindowSize.height = 0;

    self->mLastTargetArea.x = 0;
    self->mLastTargetArea.y = 0;
    self->mLastTargetArea.width = 0;
    self->mLastTargetArea.height = 0;
    self->select_node = NULL;
    self->mBlockedUpdates = NULL;
    self->mBlockedUpdatesSource = 0;
    self->prev_eventkey = NULL;
    self->is_active = TRUE;
    self->is_layout_active = TRUE;
    self->dialog_is_active = FALSE;
    self->ui_xwin_id = 0;
    self->dom_ctx->mouse_up_received = TRUE;

    // Create notification box
    self->notification_vbox = G_OBJECT(gtk_vbox_new(FALSE, 1));
    gtk_box_pack_start(GTK_BOX(self->notification_vbox), GTK_WIDGET(self->engine), TRUE, TRUE, 1);
    gtk_widget_show_all(GTK_WIDGET(self->notification_vbox));

    self->username = NULL;
    self->password = NULL;
    self->accept = FALSE;
    self->last_location = NULL;
    self->check_pending_im_enter_keyvent = FALSE;

    self->mBlockedUIRequestsSource = 0;
    self->mBlockedUIRequests = NULL;
    self->mForcedUpdateTimeout = 0;
    self->suspend_timeouts_timer_id = 0;
    self->fake_key_id = 0;
    self->timeouts_suspended = FALSE;
    self->mFixedFramesAvailable = FALSE;
    self->mPaintFixedFrames = TRUE;

    self->prev_find_str = NULL;
    self->recalculate_element = FALSE;

    self->nsteps = 0;
    self->navigation_event_id = 0;
    self->is_back = FALSE;

    return instance;
}

#ifdef HILDON_2_2

#define TREE_ITEM_LABEL 0
#define TREE_ITEM_TRUE_INDEX 1
typedef struct {
  GtkWidget* container;
  HildonTouchSelector* selector;
  GMozillaEngine* engine;
  gulong handler_id;
  GSList* items;
} SelectionData;

static 
void select_notification_selection_changed (HildonTouchSelector *selector, 
                                            gint column,
                                            SelectionData* data)
{
  ULOG_DEBUG_F();
  g_return_if_fail(selector && data && data->engine && data->items);
  
  if (data->handler_id > 0)
  {
      g_signal_handler_disconnect (selector, data->handler_id);
      data->handler_id = 0;
  }

  gint selected_item =  hildon_touch_selector_get_active(selector, column);

  GtkTreeModel* model = hildon_touch_selector_get_model(HILDON_TOUCH_SELECTOR (selector), 0);
  GtkTreeIter iter;
  int item_number;
  if (model && selected_item >= 0)
  {
    gtk_tree_model_iter_nth_child(model, &iter, NULL, selected_item);
    gint true_index = 0;
    gtk_tree_model_get(model, &iter, TREE_ITEM_TRUE_INDEX, &true_index, -1);
    item_number = ((SelectedItem*)g_slist_nth_data(data->items, true_index))->number;
    g_mozilla_select_widget_item_selected(data->engine, data->engine->select_node, &item_number, 1);
  }
  int num_items =  g_slist_length(data->items);
  int i;
  for (i = 0; i < num_items; ++i)
  {
    SelectedItem* item = g_slist_nth_data(data->items, i);
    if (item) {
      EAL_IF_GFREE(item->name);
      EAL_GFREE(item);
    }
  }
  g_slist_free(data->items);

  g_mozilla_remove_notification(data->engine, data->container);
}

static
void select_notification_done(GtkButton* button,
                              SelectionData* data)
{
  g_return_if_fail(data && data->items && data->engine);

  if (data->handler_id > 0)
  {
      g_signal_handler_disconnect (button, data->handler_id);
      data->handler_id = 0;
  }
  GList* selected = hildon_touch_selector_get_selected_rows(data->selector, 0);
  GList* item = g_list_first(selected);
  int i = 0;
  gint* selected_items = 0;
  selected_items = (gint*)g_try_malloc(sizeof(gint) * g_list_length(selected));
  while (item)
  {
    ++i;
    GtkTreePath* path = (GtkTreePath*)item->data;
    selected_items[i - 1] = ((SelectedItem*)g_slist_nth_data(data->items, gtk_tree_path_get_indices(path)[0]))->number;
    item = g_list_next(item);
  }
  g_mozilla_select_widget_item_selected (data->engine, data->engine->select_node, selected_items, i);

  int num_items =  g_slist_length(data->items);
  for (i = 0; i < num_items; ++i)
  {
    SelectedItem* item = g_slist_nth_data(data->items, i);
    if (item->name)
      g_free(item->name);
    g_free(item);
  }
  g_slist_free(data->items);

  g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
  g_list_free (selected);
  if (selected_items) g_free(selected_items);
  g_mozilla_remove_notification(data->engine, data->container);
  g_free(data);
}

#define HILDON_TOUCH_SELECTOR_WIDTH 578
#define HILDON_TOUCH_SELECTOR_HEIGHT 350
#define HILDON_DIALOG_BUTTON_WIDTH 174
#define HILDON_ELEMENT_HEIGHT 70
#define MAX_SELECTOR_BUTTONS_IN_SCREEN 5

void g_mozilla_create_select_notification(GMozillaEngine* self, GSList *list,
                                          gpointer ctx_node, gboolean multiple, const char *title, gint clicked_item)
{
  g_return_if_fail(self && list);
  GtkListStore *store_items = NULL;
  GtkCellRenderer *renderer = NULL;
  GtkWidget *selector = NULL;
  HildonTouchSelectorColumn *column = NULL;
  gint selector_height = 0;

  self->select_node = ctx_node;
  store_items = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);
  gint num_items = 0, i;
  num_items =  g_slist_length(list);

  for (i = 0; i < num_items; i++)
  {
    GtkTreeIter iter;
    SelectedItem* item = NULL;
    item = g_slist_nth_data(list, i);
    if (!item->name)
      continue;
    gtk_list_store_append(store_items, &iter);
    gtk_list_store_set(store_items, &iter, TREE_ITEM_LABEL, item->name, TREE_ITEM_TRUE_INDEX, i, -1);
  }

  // If the selection list contains less than 20 items, we use touch selector list. 
  // If more than 20 then hildon touch selector entry is used instead:
  // Note: Since entry version doesn't support multiple selection mode, normal touch
  //       selector is used in that case.
  if (multiple || num_items < 20)
  {
    selector = hildon_touch_selector_new ();
    renderer = gtk_cell_renderer_text_new ();

    column = hildon_touch_selector_append_column (HILDON_TOUCH_SELECTOR (selector),
                                                  GTK_TREE_MODEL (store_items),
                                                  renderer, "text", 0, NULL);
  }
  else
  {
    selector = hildon_touch_selector_entry_new_text ();
    hildon_touch_selector_set_model (HILDON_TOUCH_SELECTOR (selector),
                                     0,
                                     GTK_TREE_MODEL (store_items));
  }
  g_object_unref (store_items);

  g_object_set (G_OBJECT (selector), "initial-scroll", FALSE, NULL);

  // Calculate selector height:
  if ( num_items <= MAX_SELECTOR_BUTTONS_IN_SCREEN )
    selector_height = num_items * HILDON_ELEMENT_HEIGHT;
  else
    selector_height = MAX_SELECTOR_BUTTONS_IN_SCREEN * HILDON_ELEMENT_HEIGHT;

  if (multiple || num_items >= 20)
  {
    if ( multiple )
        hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
                                                   HILDON_TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE);
    else
        hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
                                                   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);

    gtk_widget_set_size_request(selector, HILDON_TOUCH_SELECTOR_WIDTH, selector_height);

    // Set the selection:
    GtkTreeModel* tree_model = hildon_touch_selector_get_model(HILDON_TOUCH_SELECTOR (selector), 0);
    GtkTreeIter iter;
    i = 0;
    if (gtk_tree_model_get_iter_first(tree_model, &iter))
    {
      do
      {
        SelectedItem* item = NULL;
        item = g_slist_nth_data(list, i);
        if (item && item->name && item->selected)
          hildon_touch_selector_select_iter(HILDON_TOUCH_SELECTOR (selector), 0, &iter, FALSE);
        else
          hildon_touch_selector_unselect_iter(HILDON_TOUCH_SELECTOR (selector), 0, &iter);
        ++i;
      }
      while (gtk_tree_model_iter_next(tree_model, &iter));
    }

    // Construct the UI and layouts:
    GtkWidget* container = gtk_hbox_new(FALSE, 0);
    GtkWidget* alignment = gtk_alignment_new (0,0.5,1,1);
    gtk_alignment_set_padding (GTK_ALIGNMENT(alignment),
                               0,
                               HILDON_MARGIN_DEFAULT,
                               HILDON_MARGIN_DOUBLE,
                               HILDON_MARGIN_DOUBLE);
    gtk_container_add (GTK_CONTAINER (alignment), GTK_WIDGET(selector));
    gtk_container_add (GTK_CONTAINER(container), alignment);

    GtkWidget *vbutton = gtk_vbutton_box_new ();
    gtk_button_box_set_layout (GTK_BUTTON_BOX(vbutton), GTK_BUTTONBOX_END);

    GtkWidget* done_button = hildon_button_new_with_text(HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL, H_(TR_WEB5_BD_DONE), NULL);
    gtk_widget_set_size_request(done_button, HILDON_DIALOG_BUTTON_WIDTH, HILDON_ELEMENT_HEIGHT);
    gtk_container_add(GTK_CONTAINER(vbutton), done_button);

    alignment = gtk_alignment_new (0.5,1,1,1);
    gtk_alignment_set_padding (GTK_ALIGNMENT(alignment),
                               0,
                               HILDON_MARGIN_DEFAULT,
                               0,
                               HILDON_MARGIN_DOUBLE);

    gtk_widget_set_size_request(alignment, HILDON_DIALOG_BUTTON_WIDTH, -1);
    gtk_container_add (GTK_CONTAINER (alignment), GTK_WIDGET(vbutton));
    gtk_container_add (GTK_CONTAINER (container), GTK_WIDGET(alignment));

    SelectionData* data = g_new0(SelectionData, 1);
    data->container = container;
    data->selector = HILDON_TOUCH_SELECTOR(selector);
    data->engine = self;
    data->items = list;
    data->handler_id = g_signal_connect(G_OBJECT(done_button), "clicked",G_CALLBACK(select_notification_done), data);

    hildon_touch_selector_center_on_index (HILDON_TOUCH_SELECTOR (selector), 0, clicked_item);
    g_mozilla_send_notification (self, container, title ? title : _(TR_WEB_LIST_SELECT_MULTI));
  }
  else
  {
    hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
                                                   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
    gtk_widget_set_size_request(selector, -1, selector_height);
    GtkTreeModel* tree_model = hildon_touch_selector_get_model(HILDON_TOUCH_SELECTOR (selector), 0);
    GtkTreeIter iter;
    i = 0;
    if (tree_model && gtk_tree_model_get_iter_first(tree_model, &iter))
    {
      do
      {
        SelectedItem* item = NULL;
        item = g_slist_nth_data(list, i);
        if (item && item->name && item->selected)
          hildon_touch_selector_select_iter(HILDON_TOUCH_SELECTOR (selector), 0, &iter, FALSE);
        else
          hildon_touch_selector_unselect_iter(HILDON_TOUCH_SELECTOR (selector), 0, &iter);
        ++i;
      }
      while (gtk_tree_model_iter_next(tree_model, &iter));
    }
    hildon_touch_selector_center_on_index (HILDON_TOUCH_SELECTOR (selector), 0, clicked_item);
    GtkWidget* alignment = gtk_alignment_new (0,0.5,1,1);
    gtk_alignment_set_padding (GTK_ALIGNMENT(alignment),
                               0,
                               HILDON_MARGIN_DEFAULT,
                               HILDON_MARGIN_DOUBLE,
                               HILDON_MARGIN_DOUBLE);
    gtk_container_add (GTK_CONTAINER (alignment), GTK_WIDGET(selector));

    SelectionData* data = g_new0(SelectionData, 1);
    data->engine = self;
    data->items = list;
    data->container = alignment;

    data->handler_id = g_signal_connect(G_OBJECT(selector), "changed",G_CALLBACK(select_notification_selection_changed), data);
    g_mozilla_send_notification (self, alignment, title ? title : _(TR_WEB_LIST_SELECT));
  }
}

#endif
